/* contact-source.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People;
using People.Utils;


namespace People.Backend.Dummy {

	public class ContactSource : Backend.BaseContactSource,
			Backend.DumpableContactSource {

		public Gee.Set<Backend.Contact> contacts {
			public get;
			private construct set;
		}

		public bool asynchronous {
			private get;
			private construct set;
		}

		public Set<DelayedOperation> delayed_operations;
		public Set<DelayedSearch> delayed_searches;

		construct {
			delayed_operations = new HashSet<DelayedOperation> ();
			delayed_searches = new HashSet<DelayedSearch> ();
			flags |= Backend.ContactSourceFlags.CAN_CREATE_CONTACT;

			this.contact_added += (s, contact) => {
				contacts.add (contact);
			};

			this.contact_removed += (s, contact) => {
				contacts.remove (contact);
			};
		}

		public ContactSource (Set<Backend.Contact> contacts, bool asynchronous) {
			this.contacts = contacts;
			this.asynchronous = asynchronous;
		}

		public bool add_contact (Backend.Contact contact) {
			foreach (Backend.Contact current in contacts) {
				if (current.id == contact.id) {
					return false;
				}
			}
			contacts.add (contact);
			register_contact (contact);
			contact_added (contact);
			return true;
		}

		public bool remove_contact (Backend.Contact contact) {
			foreach (Backend.Contact current in contacts) {
				if (current.id == contact.id) {
					contacts.remove (current);
					contact_removed (current);
					return true;
				}
			}
			return false;
		}

		public override AsyncOperation<bool> validate_contact_id (string id) {
			bool res = false;

			foreach (Backend.Contact contact in contacts) {
				if (contact.id == id) {
					res = true;
					break;
				}
			}

			if (asynchronous) {
				var op = new AsyncOperation<bool> ();
				var dop = new DelayedOperation<bool> (delayed_operations, op, res);
				delayed_operations.add (dop);
				return op;
			}

			return sync_result_bool (res);
		}

		public override Backend.Contact create_contact_object (string? id) {
			if (id != null) {
				foreach (Backend.Contact contact in contacts) {
					if (contact.id == id) {
						return contact;
					}
				}
			}

			var fields = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
			return new Contact (id, fields, contacts, asynchronous);
		}

		/* DumpableContactSource interface */
		public AsyncSearch<Backend.Contact> dump () {
			var search = new AsyncSearch<Backend.Contact> ();

			if (asynchronous) {
				var dsearch = new DelayedSearch<Backend.Contact> (delayed_searches, search, contacts);
				delayed_searches.add (dsearch);
				return search;
			}

			foreach (Backend.Contact contact in contacts) {
				search.add_hit (contact);
			}
			search.finish ();

			return search;
		}
		/* end of DumpableContactSource interface */
	}
}
