/* contact.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People;
using People.Utils;


namespace People.Backend.Dummy {

	public class Contact : Backend.BaseContact {
		private Backend.ContactFieldSpec default_field_spec =
				new ContactFieldSpec(Backend.ContactFieldFlags.WRITABLE);

		private Backend.ContactFieldSpec mandatory_field_spec =
				new ContactFieldSpec (Backend.ContactFieldFlags.WRITABLE |
									Backend.ContactFieldFlags.MANDATORY);

		public weak Gee.Set<Backend.Contact> contacts {
			private get;
			private construct set;
		}

		public Gee.Map<string, string> fields {
			private get;
			private construct set;
		}

		public bool asynchronous {
			private get;
			private construct set;
		}

		public Set<DelayedOperation> delayed_operations;

		construct {
			delayed_operations = new HashSet<DelayedOperation> ();

			flags = Backend.ContactFlags.EDITABLE | Backend.ContactFlags.DELETABLE;
		}

		public Contact (string? id, Gee.Map<string, string> fields,
				Gee.Set<Backend.Contact> contacts, bool asynchronous) {
			this.id = id;
			this.fields = fields;
			this.contacts = contacts;
			this.asynchronous = asynchronous;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields () {
			var fields = fields.get_keys ();

			if (asynchronous) {
				var op = new AsyncOperation<Gee.Set<string> > ();
				var dop = new DelayedOperation<Gee.Set<string> > (delayed_operations, op, fields);
				delayed_operations.add (dop);
				return op;
			}

			return sync_result (fields);
		}

		public override AsyncOperation<Gee.Set<string> > get_definable_fields () {
			var fields = new Gee.HashSet<string> (str_hash, str_equal);

			if (asynchronous) {
				var op = new AsyncOperation<Gee.Set<string> > ();
				var dop = new DelayedOperation<Gee.Set<string> > (delayed_operations, op, fields);
				delayed_operations.add (dop);
				return op;
			}

			return sync_result (fields);
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields () {
			var fields = new Gee.HashSet<string> (str_hash, str_equal);
			fields.add ("identity.name.first");
			fields.add ("identity.name.last");

			if (asynchronous) {
				var op = new AsyncOperation<Gee.Set<string> > ();
				var dop = new DelayedOperation<Gee.Set<string> > (delayed_operations, op, fields);
				delayed_operations.add (dop);
				return op;
			}

			return sync_result (fields);
		}

		public override AsyncOperation<ContactFieldSpec> get_field_spec (string field_name) {
			Backend.ContactFieldSpec spec;

			if (field_name == "identity.name.first" || field_name == "identity.name.last") {
				spec = mandatory_field_spec;
			} else {
				spec = default_field_spec;
			}

			if (asynchronous) {
				var op = new AsyncOperation<Backend.ContactFieldSpec> ();
				var dop = new DelayedOperation<Backend.ContactFieldSpec> (delayed_operations, op, spec);
				delayed_operations.add (dop);
				return op;
			}

			return sync_result (spec);
		}

		public override AsyncOperation<Gee.Map<string, string>> get_fields (Gee.Set<string> field_names) {
			var fields_result = new HashMap<string, string> (str_hash, str_equal, str_equal);

			foreach (string name in field_names) {
				if (fields.contains (name)) {
					fields_result.set (name, fields.get (name));
				}
			}

			if (asynchronous) {
				var op = new AsyncOperation<Gee.Map<string, string> > ();
				var dop = new DelayedOperation<Gee.Map<string, string> > (delayed_operations, op, fields_result);
				delayed_operations.add (dop);
				return op;
			}

			return sync_result (fields_result);
		}

		public override AsyncOperation<Gee.Set<string> > set_fields (Gee.Map<string, string> fields) {
			if (id == null) {
				if (!fields.contains ("identity.name.first") || !fields.contains ("identity.name.last")) {
					return sync_error (new Backend.ContactError.MISSING_MANDATORY_FIELD (""));
				}
				id = fields.get ("identity.name.first") + "-" + fields.get ("identity.name.last");
			}

			foreach (string name in fields.get_keys ()) {
				this.fields.set (name, fields.get (name));
			}

			var fields_result = fields.get_keys ();

			if (asynchronous) {
				var op = new AsyncOperation<Gee.Set<string> > ();
				var dop = new DelayedOperation<Gee.Set<string> > (delayed_operations, op, fields_result);
				delayed_operations.add (dop);
				return op;
			}

			return sync_result (fields_result);
		}

		public override AsyncOperation<bool> @delete () {
			var res = contacts.remove (this);

			if (asynchronous) {
				var op = new AsyncOperation<bool> ();
				var dop = new DelayedOperation<bool> (delayed_operations, op, res);
				delayed_operations.add (dop);
				return op;
			}

			return sync_result_bool (res);
		}

	}

}
