/* contact.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;
using Soup;
using Xml;

using People;
using People.Utils;


namespace People.Backend.Friendfeed {

	public class Contact : Backend.BaseContact {
		private Map<Soup.Message, AsyncOperation> pending_operations =
				new HashMap<Soup.Message, AsyncOperation> ();
		private Map<Soup.Message, Gee.Set<string> > pending_requests_fields =
				new HashMap<Soup.Message, Gee.Set<string> > ();

		private static Backend.ContactFieldSpec default_field_spec =
				new Backend.ContactFieldSpec (Backend.ContactFieldFlags.NONE);

		private Soup.Session session;

		construct {
			flags = Backend.ContactFlags.NONE;

			session = new Soup.SessionAsync ();
		}

		public Contact (string id) {
			this.id = id;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields () {
			var operation = async_operation_init ();

			var url = "http://friendfeed.com/api/user/%s/profile?format=xml".printf (id);
			var msg = new Soup.Message ("GET", url);
			pending_operations.set (msg, operation);

			session.queue_message (msg, get_defined_fields_callback);
			return operation;
		}

		private void get_defined_fields_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);

			if (msg.status_code != KnownStatusCode.OK) {
				async_operation_error (operation, new Backend.Error.INTERNAL_ERROR ("User does not exist"));
				pending_operations.remove (msg);
				return;
			}

			var fields = new Gee.HashSet<string> (str_hash, str_equal);

			Xml.Doc* doc = Xml.Parser.parse_memory (msg.response_body.data,
					(int) msg.response_body.length);
			if (doc != null && doc->children != null) {
				for (Xml.Node* node = doc->children->children; node != null; node = node->next) {
					if (node->name == "service") {
						string service = null;
						var current = new Gee.HashSet<string> (str_hash, str_equal);
						for (Xml.Node* entry = node->children; entry != null; entry = entry->next) {
							if (entry->name == "id") {
								service = entry->get_content ();
							} else if (entry->name == "username") {
								current.add ("user");
							} else if (entry->name == "profileUrl") {
								current.add ("url");
							} else if (entry->name == "iconUrl") {
								current.add ("icon");
							}
						}

						if (service != null) {
							foreach (string type in current) {
								fields.add ("communication.web.%s.%s".printf (service, type));
							}
						}
					} else if (node->name == "name") {
						fields.add ("identity.name.full");
					} else if (node->name == "nickname") {
						fields.add ("communication.web.friendfeed.user");
						fields.add ("communication.web.friendfeed.url");
						fields.add ("identity.name.nickname");
					} else if (node->name == "id") {
						fields.add ("communication.web.friendfeed.avatar");
					}
				}
			}

			delete doc;

			async_operation_emit (operation, fields);
			pending_operations.remove (msg);
		}

		public override AsyncOperation<Backend.ContactFieldSpec> get_field_spec (string field_name) {
			return sync_result (default_field_spec);
		}

		public override AsyncOperation<Gee.ReadOnlyMap<string, string> >
				get_fields (Gee.Set<string> fields) {
			var operation = async_operation_init ();

			var url = "http://friendfeed.com/api/user/%s/profile?format=xml".printf (id);
			var msg = new Soup.Message ("GET", url);
			pending_operations.set (msg, operation);
			pending_requests_fields.set (msg, fields);

			session.queue_message (msg, get_fields_callback);
			return operation;
		}

		private void get_fields_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);
			var fields = pending_requests_fields.get (msg);
			var values = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);

			if (msg.status_code != KnownStatusCode.OK) {
				async_operation_error (operation, new Backend.Error.INTERNAL_ERROR ("User does not exist"));
				pending_operations.remove (msg);
				return;
			}

			Xml.Doc* doc = Xml.Parser.parse_memory (msg.response_body.data,
					(int) msg.response_body.length);
			if (doc != null && doc->children != null) {
				for (Xml.Node* node = doc->children->children; node != null; node = node->next) {
					if (node->name == "service") {
						string service = null;
						var current = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
						for (Xml.Node* entry = node->children; entry != null; entry = entry->next) {
							if (entry->name == "id") {
								service = entry->get_content ();
							} else if (entry->name == "username") {
								current.set ("user", entry->get_content ());
							} else if (entry->name == "profileUrl") {
								current.set ("url", entry->get_content ());
							} else if (entry->name == "iconUrl") {
								current.set ("icon", entry->get_content ());
							}
						}

						if (service != null) {
							foreach (string type in current.get_keys ()) {
								var ns = "communication.web.%s.%s".printf (service, type);
								if (fields.contains (ns)) {
									values.set (ns, current.get (type));
								}
							}
						}
					} else if (node->name == "name") {
						if (fields.contains ("identity.name.full")) {
							values.set ("identity.name.full", node->get_content ());
						}
					} else if (node->name == "id") {
						if (fields.contains ("communication.web.friendfeed.avatar")) {
							var avatar = "http://friendfeed.s3.amazonaws.com/pictures-%s-medium.jpg".printf (node->get_content ());
							values.set ("communication.web.friendfeed.avatar", avatar);
						}
					} else if (node->name == "nickname") {
						if (fields.contains ("communication.web.friendfeed.user")) {
							values.set ("communication.web.friendfeed.user", node->get_content ());
						}
						if (fields.contains ("communication.web.friendfeed.url")) {
							values.set ("communication.web.friendfeed.url", 
										"http://friendfeed.com/" + node->get_content ());
						}
						if (fields.contains ("identity.name.nickname")) {
							values.set ("identity.name.nickname", node->get_content ());
						}
					}
				}
			}

			delete doc;

			async_operation_emit (operation, values);
			pending_operations.remove (msg);
			pending_requests_fields.remove (msg);
		}

		/* this contact source is read-only */
		public override AsyncOperation<Gee.Set<string> > get_definable_fields () {
			return sync_result (new Gee.HashSet<string>(str_hash, str_equal));
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields () {
			return sync_result (new Gee.HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<Gee.ReadOnlySet<string> >
				set_fields (Gee.Map<string, string> fields) {
			return sync_result (new Gee.HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<bool> @delete() {
			return sync_result_bool (false);
		}
	}

}
