/* contact-source.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;
using Soup;
using Json;

using People;
using People.Utils;


namespace People.Backend.Google {

	public class ContactSource : Backend.BaseContactSource,
			Backend.DumpableContactSource {

		private Map<Message, AsyncSearch> pending_searches =
				new HashMap<Message, AsyncSearch> ();
		private Map<Message, AsyncOperation> pending_operations =
				new HashMap<Message, AsyncOperation> ();

		private Soup.Session session = new Soup.SessionAsync ();

		public string authentication_token {
			private get;
			construct;
		}

		public ContactSource (string authentication_token) {
			this.authentication_token = authentication_token;
			this.flags = Backend.ContactSourceFlags.NEED_NETWORK;
		}

		public override Backend.Contact create_contact_object (string? id) {
			return new Contact (id, authentication_token);
		}

		public override AsyncOperation<bool> validate_contact_id (string id) {
			var operation = async_operation_init_bool ();
			var url = "http://www.google.com/m8/feeds/contacts/default/base/%s?alt=json".printf (id);
			var msg = new Soup.Message ("GET", url);
			msg.request_headers.append ("Authorization",
					"GoogleLogin auth=%s".printf (authentication_token));
			pending_operations.set (msg, operation);
			session.queue_message (msg, validate_contact_id_callback);

			return operation;
		}

		private void validate_contact_id_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);
			pending_operations.remove (msg);

			if (msg.status_code == KnownStatusCode.OK) {
				async_operation_emit_bool (operation, true);
			} else {
				async_operation_emit_bool (operation, false);
			}
		}

		/* DumpableContactSource interface */
		public AsyncSearch<Backend.Contact> dump() {
			var search = new AsyncSearch<Backend.Contact> ();

			var url = "http://www.google.com/m8/feeds/contacts/default/base?alt=json";
			var msg = new Soup.Message ("GET", url);
			msg.request_headers.append ("Authorization",
					"GoogleLogin auth=%s".printf (authentication_token));
			pending_searches.set (msg, search);
			session.queue_message (msg, dump_callback);

			return search;
		}

		private void dump_callback (Session session, Message msg) {
			var search = pending_searches.get (msg);
			pending_searches.remove (msg);

			if (msg.status_code == KnownStatusCode.OK) {
				var parser = new Parser ();
				parser.load_from_data (msg.response_body.data,
						(ulong) msg.response_body.data.size);

				weak Json.Object root = parser.get_root ().get_object ();
				weak Json.Object feed = root.get_member ("feed").get_object ();
				weak Json.Array entries = feed.get_member ("entry").get_array ();

				for (uint i = 0; i < entries.get_length (); i++) {
					weak Json.Object entry = entries.get_element (i).get_object ();

					weak Json.Object id_obj = entry.get_member ("id").get_object ();
					string id = id_obj.get_member ("$t").get_string ();

					string[] list = id.split ("/");
					var contact = get_contact_object (list[8]);
					search.add_hit (contact);
				}
			}

			search.finish ();
		}
		/* end of DumpableContactSource interface */
	}
}
