/* contact.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;
using Soup;
using Json;

using People;
using People.Utils;


namespace People.Backend.Google {

	public class Contact : Backend.BaseContact {
		private Map<Soup.Message, AsyncOperation> pending_operations;
		private Map<Soup.Message, Gee.Set<string> > pending_requests_fields;

		private Soup.Session session;

		private Backend.ContactFieldSpec default_field_spec =
				new ContactFieldSpec(Backend.ContactFieldFlags.WRITABLE);

		public string authentication_token {
			private get;
			private construct set;
		}

		construct {
			pending_operations = new HashMap<Soup.Message, AsyncOperation> ();
			pending_requests_fields = new HashMap<Soup.Message, Gee.Set<string> > ();
			session = new Soup.SessionAsync ();

			flags = Backend.ContactFlags.NONE;
		}

		public Contact (string? id, string authentication_token) {
			this.id = id;
			this.authentication_token = authentication_token;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields () {
			var operation = async_operation_init ();

			var url = "http://www.google.com/m8/feeds/contacts/default/full/%s?alt=json".printf (id);
			var msg = new Soup.Message ("GET", url);
			msg.request_headers.append ("Authorization",
					"GoogleLogin auth=%s".printf (authentication_token));
			pending_operations.set (msg, operation);

			session.queue_message (msg, get_defined_fields_callback);
			return operation;
		}

		private void get_defined_fields_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);

			if (msg.status_code != KnownStatusCode.OK) {
				async_operation_error (operation, new Backend.Error.INTERNAL_ERROR ("User does not exist"));
				pending_operations.remove (msg);
				return;
			}

			var parser = new Parser ();
			parser.load_from_data (msg.response_body.data,
						(ulong) msg.response_body.data.size);
			var fields = extract_fields (parser).get_keys ();

			async_operation_emit (operation, fields);
			pending_operations.remove (msg);
		}

		public override AsyncOperation<Gee.Map<string, string> >
				get_fields (Gee.Set<string> field_names) {
			var operation = async_operation_init ();

			var url = "http://www.google.com/m8/feeds/contacts/default/full/%s?alt=json".printf (id);
			var msg = new Soup.Message ("GET", url);
			msg.request_headers.append ("Authorization",
					"GoogleLogin auth=%s".printf (authentication_token));
			pending_operations.set (msg, operation);
			pending_requests_fields.set (msg, field_names);
			session.queue_message (msg, get_fields_callback);
			return operation;
		}

		private void get_fields_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);
			var requested_fields = pending_requests_fields.get (msg);

			pending_operations.remove (msg);
			pending_requests_fields.remove (msg);

			if (msg.status_code != KnownStatusCode.OK) {
				async_operation_error (operation, new Backend.Error.INTERNAL_ERROR ("User does not exist"));
				pending_operations.remove (msg);
				return;
			}

			var parser = new Parser ();
			parser.load_from_data (msg.response_body.data,
						(ulong) msg.response_body.data.size);

			var defined_fields = extract_fields (parser);
			var fields = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
			foreach (string field_name in requested_fields) {
				if (defined_fields.contains (field_name)) {
					fields.set (field_name, defined_fields.get (field_name));
				}
			}
			async_operation_emit (operation, fields);
		}

		public override AsyncOperation<ContactFieldSpec> get_field_spec (string field_name) {
			return sync_result (default_field_spec);
		}

		public override AsyncOperation<Gee.Set<string> > get_definable_fields () {
			return sync_result (new Gee.HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields () {
			return sync_result (new Gee.HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<Gee.Set<string> > set_fields (Gee.Map<string, string> fields) {
			return sync_result (new Gee.HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<bool> @delete () {
			return sync_result_bool (false);
		}


		private Gee.Map<string, string> extract_fields (Parser parser) {
			weak Json.Object root = parser.get_root ().get_object ();
			weak Json.Object entry = root.get_member ("entry").get_object ();

			var fields = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);

			weak Json.Node node;
			weak Json.Array property;
			weak Json.Object item;

			/* full name */
			node = entry.get_member ("title");
			if (node != null) {
				var full = node.get_object ().get_member ("$t").get_string ();
				fields.set ("identity.name.full", full);
			}

			/* organization */
			node = entry.get_member ("gd$organization");
			if (node != null) {
				property = node.get_array ();

				for (uint i = 0; i < property.get_length (); i++) {
					item = property.get_element (i).get_object ();

					/* organization */
					string org = "";
					weak Json.Node tmp = item.get_member ("gd$orgName");
					if (tmp != null) {
						org = tmp.get_object ().get_member ("$t").get_string ();
						fields.set ("activity.organization", org);
					}

					/* title */
					string title = "";
					tmp = item.get_member ("gd$orgTitle");
					if (tmp != null) {
						title = tmp.get_object ().get_member ("$t").get_string ();
						fields.set ("activity.title", title);
					}
				}
			}

			/* email */
			node = entry.get_member ("gd$email");
			if (node != null) {
				property = node.get_array ();

				for (uint i = 0; i < property.get_length (); i++) {
					item = property.get_element (i).get_object ();

					/* location */
					string location = "";
					var rel = item.get_member ("rel").get_string ().split ("#");
					location = "/" + rel[1];

					/* address */
					string address = "";
					weak Json.Node tmp = item.get_member ("address");
					if (tmp != null) {
						address = tmp.get_string ();
						fields.set ("communication.email%s".printf (location), address);
					}
				}
			}

			/* im */
			node = entry.get_member ("gd$im");
			if (node != null) {
				property = node.get_array ();

				for (uint i = 0; i < property.get_length (); i++) {
					item = property.get_element (i).get_object ();

					/* protocol */
					string protocol = "";
					var rel = item.get_member ("protocol").get_string ().split ("#");
					protocol = "." + rel[1].down ((long) rel[1].size);

					/* account */
					string address = "";
					weak Json.Node tmp = item.get_member ("address");
					if (tmp != null) {
						address = tmp.get_string ();
						fields.set ("communication.im%s".printf (protocol), address);
					}
				}
			}

			/* phone */
			node = entry.get_member ("gd$phoneNumber");
			if (node != null) {
				property = node.get_array ();

				for (uint i = 0; i < property.get_length (); i++) {
					item = property.get_element (i).get_object ();

					/* system */
					string field = "";
					var rel = item.get_member ("rel").get_string ().split ("#");
					if (rel[1] == "home_fax") {
						field = "communication.fax/home";
					} else if (rel[1] == "pager") {
						field = "communication.pager";
					} else if (rel[1] == "work_fax") {
						field = "communication.fax/work";
					} else {
						field = "communication.phone/%s".printf (rel[1]);
					}

					/* number */
					string number = "";
					weak Json.Node tmp = item.get_member ("$t");
					if (tmp != null) {
						number = tmp.get_string ();
						fields.set (field, number);
					}
				}
			}

			/* postal address */
			node = entry.get_member ("gd$postalAddress");
			if (node != null) {
				property = node.get_array ();

				for (uint i = 0; i < property.get_length (); i++) {
					item = property.get_element (i).get_object ();

					/* location */
					string location = "";
					var rel = item.get_member ("rel").get_string ().split ("#");
					location = "/" + rel[1];

					/* address */
					string address = "";
					weak Json.Node tmp = item.get_member ("$t");
					if (tmp != null) {
						address = tmp.get_string ();
						fields.set ("communication.address%s".printf (location), address);
					}
				}
			}

			return fields;
		}

	}

}
