/* contact-source.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;
using Soup;

using People;
using People.Utils;


namespace People.Backend.Lastfm {

	public class ContactSource : Backend.BaseContactSource,
			Backend.DumpableContactSource {

		private Map<Message, AsyncSearch> pending_searches =
				new HashMap<Message, AsyncSearch> ();

		private Map<Message, AsyncOperation> pending_operations =
				new HashMap<Message, AsyncOperation> ();
		private Map<Message, string> pending_contact_requests =
				new HashMap<Message, string> (direct_hash, direct_equal, str_equal);

		private Soup.Session session = new Soup.SessionAsync ();

		public string user {
			get;
			construct;
		}

		public ContactSource (string user) {
			this.user = user;
			this.flags = Backend.ContactSourceFlags.NEED_NETWORK;
		}

		public override AsyncOperation<bool> validate_contact_id (string id) {
			var operation = async_operation_init_bool ();
			var url = "http://ws.audioscrobbler.com/1.0/user/%s/friends.txt".printf (user);
			var msg = new Soup.Message ("GET", url);
			pending_operations.set (msg, operation);
			pending_contact_requests.set (msg, id);
			session.queue_message (msg, validate_contact_id_callback);

			return operation;
		}

		public override Backend.Contact create_contact_object (string? id) {
			return new Contact (id);
		}

		private void validate_contact_id_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);
			var id = pending_contact_requests.get (msg);
			pending_contact_requests.remove (msg);
			pending_operations.remove (msg);

			if (msg.response_body.data != null && msg.response_body.data != "No user exists with this name.") {
				var friends = msg.response_body.data.split ("\n");
				foreach (string friend in friends) {
					if (friend == id) {
						async_operation_emit_bool (operation, true);
						return;
					}
				}
			} else {
				async_operation_error (operation, new Backend.Error.INVALID_ARGUMENT ("User does not exist: " + id));
				return;
			}

			async_operation_emit_bool (operation, false);
		}

		/* DumpableContactSource interface */
		public AsyncSearch<Backend.Contact> dump() {
			var search = new AsyncSearch<Backend.Contact>();

			var url = "http://ws.audioscrobbler.com/1.0/user/%s/friends.txt".printf (user);
			var msg = new Soup.Message ("GET", url);
			pending_searches.set (msg, search);
			session.queue_message (msg, dump_callback);

			return search;
		}

		private void dump_callback (Session session, Message msg) {
			var search = pending_searches.get (msg);

			if (msg.response_body.data != null && msg.response_body.data != "No user exists with this name.") {
				var friends = msg.response_body.data.split ("\n");
				foreach (string friend in friends) {
					if (friend != "") {
						var contact = get_contact_object (friend);
						search.add_hit(contact);
					}
				}
			}

			search.finish ();
			pending_searches.remove (msg);
		}
		/* end of DumpableContactSource interface */
	}
}
