/* contact.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;
using Soup;
using Xml;

using People;
using People.Utils;


namespace People.Backend.Lastfm {

	public class Contact : Backend.BaseContact {
		private Map<Soup.Message, AsyncOperation> pending_operations =
				new HashMap<Soup.Message, AsyncOperation> ();
		private Map<Soup.Message, Gee.Set<string> > pending_requests_fields =
				new HashMap<Soup.Message, Gee.Set<string> > ();

		private static ReadOnlyMap<string, string> ns_to_elmt = null;
		private static ReadOnlyMap<string, string> elmt_to_ns = null;

		private static Backend.ContactFieldSpec default_field_spec =
				new Backend.ContactFieldSpec (Backend.ContactFieldFlags.NONE);

		private Soup.Session session;

		construct {
			var ns_to_elmt = new HashMap<string, string>(str_hash, str_equal, str_equal);
			ns_to_elmt.set("identity.name.full", "realname");
			ns_to_elmt.set("identity.gender", "gender");
			ns_to_elmt.set("identity.age", "age");
			ns_to_elmt.set("identity.country", "country");
			ns_to_elmt.set("communication.web.lastfm.url", "url");
			ns_to_elmt.set("communication.web.lastfm.avatar", "avatar");
			this.ns_to_elmt = new ReadOnlyMap<string, string>(ns_to_elmt);

			var elmt_to_ns = new HashMap<string, string>(str_hash, str_equal, str_equal);
			elmt_to_ns.set("realname", "identity.name.full");
			elmt_to_ns.set("gender", "identity.gender");
			elmt_to_ns.set("age", "identity.age");
			elmt_to_ns.set("country", "identity.country");
			elmt_to_ns.set("url", "communication.web.lastfm.url");
			elmt_to_ns.set("avatar", "communication.web.lastfm.avatar");
			this.elmt_to_ns = new ReadOnlyMap<string, string>(elmt_to_ns);

			flags = Backend.ContactFlags.NONE;

			session = new Soup.SessionAsync ();
		}

		public Contact (string id) {
			this.id = id;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields () {
			var operation = async_operation_init ();

			var url = "http://ws.audioscrobbler.com/1.0/user/%s/profile.xml".printf (id);
			var msg = new Soup.Message ("GET", url);
			pending_operations.set (msg, operation);

			session.queue_message (msg, get_defined_fields_callback);
			return operation;
		}

		private void get_defined_fields_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);

			var fields = new Gee.HashSet<string> (str_hash, str_equal);
			fields.add ("communication.web.lastfm.user");

			if (msg.response_body.data == "No user exists with this name.") {
				async_operation_error (operation, new Backend.Error.INTERNAL_ERROR ("User does not exist"));
				pending_operations.remove (msg);
				return;
			} else {
				weak Xml.Doc doc = Xml.Parser.parse_memory (msg.response_body.data,
						(int) msg.response_body.length);
				if (doc.children != null) {
					Xml.Node *node;
					for (node = doc.children->children; node != null; node = node->next) {
						var ns = elmt_to_ns.get (node->name);
						if (ns != null) {
							fields.add (ns);
						}
					}
				}
			}

			async_operation_emit (operation, fields);
			pending_operations.remove (msg);
		}

		public override AsyncOperation<Backend.ContactFieldSpec> get_field_spec (string field_name) {
			return sync_result (default_field_spec);
		}

		public override AsyncOperation<Gee.ReadOnlyMap<string, string> >
				get_fields (Gee.Set<string> fields) {
			var operation = async_operation_init ();

			var url = "http://ws.audioscrobbler.com/1.0/user/%s/profile.xml".printf (id);
			var msg = new Soup.Message ("GET", url);
			pending_operations.set (msg, operation);
			pending_requests_fields.set (msg, fields);

			session.queue_message (msg, get_fields_callback);
			return operation;
		}

		private void get_fields_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);
			var fields = pending_requests_fields.get (msg);
			var values = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);

			if (fields.contains ("communication.web.lastfm.user")) {
				values.set ("communication.web.lastfm.user", id);
			}

			if (msg.response_body.data == "No user exists with this name.") {
				async_operation_error (operation, new Backend.Error.INTERNAL_ERROR ("User does not exist"));
				pending_operations.remove (msg);
				pending_requests_fields.remove (msg);
				return;
			} else {
				weak Xml.Doc doc = Xml.Parser.parse_memory (msg.response_body.data,
						(int) msg.response_body.length);
				if (doc != null && doc.children != null) {
					Xml.Node *node;
					for (node = doc.children->children; node != null; node = node->next) {
						var ns = elmt_to_ns.get (node->name);
						if (ns != null && fields.contains (ns)) {
							values.set (ns, node->get_content ());
						}
					}
				}
			}

			async_operation_emit (operation, values);
			pending_operations.remove (msg);
			pending_requests_fields.remove (msg);
		}

		/* this contact source is read-only */
		public override AsyncOperation<Gee.Set<string> > get_definable_fields () {
			return sync_result (new Gee.HashSet<string>(str_hash, str_equal));
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields () {
			return sync_result (new Gee.HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<Gee.ReadOnlySet<string> >
				set_fields (Gee.Map<string, string> fields) {
			return sync_result (new Gee.HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<bool> @delete() {
			return sync_result_bool (false);
		}
	}

}
