/* contact-source.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;
using Sqlite;

using People;
using People.Utils;


namespace People.Backend.Sqlite {
	public class ContactSource : Backend.BaseContactSource,
			Backend.DumpableContactSource,
			Backend.QueryableContactSource {

		public string db_path {
			private get;
			construct;
		}

		private global::Sqlite.Database db;
		private AsyncOperation<bool> pending_id_validation;
		private AsyncSearch<Contact> pending_query;

		construct {
			DirUtils.create_with_parents (Path.get_dirname (db_path), 0700);
			if (global::Sqlite.Database.open (db_path, out db) != global::Sqlite.OK) {
				warning ("Database '%s' open failed : %s", db_path, db.errmsg ());
				db = null;
			} else {
				db.exec (
					"CREATE TABLE IF NOT EXISTS people(" +
						"id integer PRIMARY KEY AUTOINCREMENT," +
						"first_name string," +
						"middle_name string," +
						"last_name string," +
						"home_phone string," +
						"work_phone string," +
						"mobile_phone string," +
						"home_email string," +
						"work_email string" +
					");");
			}
			flags |= Backend.ContactSourceFlags.CAN_CREATE_CONTACT;
		}

		public ContactSource (string db_path) {
			this.db_path = db_path;
		}

		public override AsyncOperation<bool> validate_contact_id (string id) {
			if (!Regex.match_simple ("^[0-9]+$", id)) {
				return sync_result_bool (false);
			}

			var op = async_operation_init_bool ();
			pending_id_validation = op;

			var errcode = db.exec ("SELECT count(*) FROM people WHERE id=%s".printf (id),
					validate_contact_id_callback);
			if (errcode != global::Sqlite.OK) {
				async_operation_error (op, new Backend.Error.INTERNAL_ERROR (db.errmsg()));
			}

			return op;
		}

		public override Backend.Contact create_contact_object (string? id) {
			return new Contact (id, db);
		}

		private int validate_contact_id_callback (int n_columns, string[] values, string[] column_names) {
			var op = pending_id_validation;
			pending_id_validation = null;

			this.async_operation_emit_bool (op, values[0].to_int() > 0);

			return 0;
		}

		/* DumpableContactSource interface */
		public AsyncSearch<Contact> dump () {
			return query (null);
		}
		/* end of DumpableContactSource interface */

		/* QueryableContactSource interface */
		public AsyncSearch<Contact> query (Query? query) requires (pending_query == null) {
			var search = new AsyncSearch<Contact> ();
			var visitor = new SQLQueryVisitor ();

			pending_query = search;
			db.exec (visitor.translate (query), query_callback);
			pending_query = null;
			search.finish();

			return search;
		}

		private int query_callback(int n_columns, string[] values, string[] column_names) requires (pending_query != null) {
			AsyncSearch search = pending_query;

			for (int i = 0; i < n_columns; i++) {
				if (column_names[i] == "id") {
					string id = values[i];
					var contact = get_contact_object (id);
					search.add_hit (contact);
				}
			}
			return 0;
		}
		/* end of QueryableContactSource interface */
	}
}
