/* contact.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People;
using People.Utils;


namespace People.Backend.Sqlite {

	public class Contact : Backend.BaseContact {
		private static ReadOnlyMap<string, string> ns_to_col = null;
		private static ReadOnlyMap<string, string> col_to_ns = null;

		private static Backend.ContactFieldSpec _default_field_spec =
				new Backend.ContactFieldSpec (Backend.ContactFieldFlags.WRITABLE);

		private Gee.List<AsyncOperation<Gee.Set<string>>> pending_get_definable_fields;
		private Gee.List<AsyncOperation<Gee.Map<string, string>>> pending_get_fields;

		public weak global::Sqlite.Database db {
			private get;
			construct;
		}

		construct {
			var ns_to_col = new HashMap<string, string> (str_hash, str_equal, str_equal);
			ns_to_col.set ("identity.name.first", "first_name");
			ns_to_col.set ("identity.name.middle", "middle_name");
			ns_to_col.set ("identity.name.last", "last_name");
			ns_to_col.set ("communication.phone.home", "home_phone");
			ns_to_col.set ("communication.phone.work", "work_phone");
			ns_to_col.set ("communication.phone.mobile", "mobile_phone");
			ns_to_col.set ("communication.email.home", "home_email");
			ns_to_col.set ("communication.email.work", "work_email");
			this.ns_to_col = new ReadOnlyMap<string, string> (ns_to_col);

			var col_to_ns = new HashMap<string, string> (str_hash, str_equal, str_equal);
			col_to_ns.set ("first_name","identity.name.first");
			col_to_ns.set ("middle_name", "identity.name.middle");
			col_to_ns.set ("last_name","identity.name.last");
			col_to_ns.set ("home_phone", "communication.phone.home");
			col_to_ns.set ("work_phone", "communication.phone.work");
			col_to_ns.set ("mobile_phone", "communication.phone.mobile");
			col_to_ns.set ("home_email","communication.email.home");
			col_to_ns.set ("work_email","communication.email.work");
			this.col_to_ns = new ReadOnlyMap<string, string> (col_to_ns);

			pending_get_definable_fields = new Gee.ArrayList<AsyncOperation<Gee.Set<string>>> ();
			pending_get_fields = new Gee.ArrayList<AsyncOperation<Gee.Map<string, string>>> ();
			flags = ContactFlags.EDITABLE | ContactFlags.DELETABLE;
		}

		public Contact (string? id, global::Sqlite.Database db) {
			this.id = id;
			this.db = db;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields () {
			var op = async_operation_init ();

			if (id == null) {
				async_operation_emit (op, new HashSet<string> (str_hash, str_equal));
			} else {
				pending_get_definable_fields.add (op);
				var errcode = db.exec ("SELECT * FROM people WHERE id=%s".printf (id),
						get_defined_fields_callback);
				if (errcode != global::Sqlite.OK)
					async_operation_error (op, new Error.INTERNAL_ERROR (db.errmsg()));
			}

			return op;
		}

		private int get_defined_fields_callback (int n_columns, string[] values, string[] column_names) requires (pending_get_definable_fields.size > 0) {
			var op = pending_get_definable_fields.get (0);
			pending_get_definable_fields.remove_at (0);

			var fields = new Gee.HashSet<string> (str_hash, str_equal);
			for (int i = 0; i < n_columns; i++) {
				if (column_names[i] != "id" && values[i] != null) {
					fields.add (col_to_ns.get (column_names[i]));
				}
			}

			async_operation_emit (op, fields);
			return 0;
		}

		public override AsyncOperation<Gee.Set<string> > get_definable_fields () {
			return sync_result (ns_to_col.get_keys ());
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields () {
			return sync_result (new HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<ContactFieldSpec> get_field_spec (string field_name) {
			return sync_result (_default_field_spec);
		}

		public override AsyncOperation<Gee.Set<string>> set_fields (Gee.Map<string, string> fields) {
			string query;

			Gee.Set<string> updated_fields = new Gee.HashSet<string> (str_hash, str_equal);
			if (id == null) {
				var columns = ""; var values = "";
				foreach (string field in fields.get_keys ()) {
					if (!ns_to_col.contains (field))
						continue;

					updated_fields.add (field);

					if (columns == "") {
						columns = ns_to_col.get (field);
						values = "\"%s\"".printf (fields.get (field));
					} else {
						columns = "%s, %s".printf (columns, ns_to_col.get (field));
						values = "%s, \"%s\"".printf (values, fields.get (field));
					}
				}

				query = "INSERT INTO people (%s) VALUES (%s)".printf (columns, values);
			} else {
				var assignments = "";
				foreach (string field in fields.get_keys ()) {
					if (!ns_to_col.contains (field)) {
						continue;
					}

					updated_fields.add (field);

					if (assignments == "") {
						assignments = "%s=\"%s\"".printf (ns_to_col.get (field),fields.get (field));
					} else {
						assignments = "%s, %s=\"%s\"".printf (assignments, ns_to_col.get (field),
							fields.get (field));
					}
				}

				query = "UPDATE people SET %s WHERE id=%s".printf (assignments, id);
			}

			AsyncOperation op;
			if (db.exec (query) == global::Sqlite.OK) {
				if (id == null) {
					id = db.last_insert_rowid ().to_string ();
				}

				op = sync_result (updated_fields);
			} else {
				op = sync_error (new Backend.Error.INTERNAL_ERROR (db.errmsg ()));
			}
			return op;
		}

		public override AsyncOperation<Gee.Map<string, string> > get_fields (Gee.Set<string> fields) {
			var op = async_operation_init ();

			if (id == null) {
				var empty_result = new Gee.HashMap<string, string> (str_hash,
						str_equal, str_equal);
				op.completed_with_result (empty_result);
			} else {
				var columns = "";
				foreach (string field in fields) {
					if (!ns_to_col.contains(field))
						continue;

					if (columns == "") {
						columns = ns_to_col.get (field);
					} else {
						columns = "%s, %s ".printf (columns, ns_to_col.get (field));
					}
				}

				pending_get_fields.add (op);
				var errcode = db.exec ("SELECT %s FROM people WHERE id=%s".printf (columns, id), get_fields_callback);
				if (errcode != global::Sqlite.OK)
					async_operation_error (op, new Backend.Error.INTERNAL_ERROR (db.errmsg ()));

				if (op.completed == false) {
					var empty_result = new Gee.HashMap<string, string> (str_hash,
							str_equal, str_equal);
					op.completed_with_result (empty_result);
				}
			}
			return op;
		}

		private int get_fields_callback (int n_columns, string[] values, string[] column_names) requires (pending_get_fields.size > 0) {
			var op = pending_get_fields.get (0);
			pending_get_fields.remove_at (0);

			var fields = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
			for (int i = 0; i < n_columns; i++) {
				if (values[i] != null)
					fields.set (col_to_ns.get (column_names[i]), values[i]);
			}

			async_operation_emit (op, fields);
			return 0;
		}

		public override AsyncOperation<bool> @delete () {
			AsyncOperation op;
			if (id == null) {
				op = sync_result_bool (false);
			} else {
				var errcode = db.exec ("DELETE FROM people WHERE id=%s".printf(id));

				if (errcode == global::Sqlite.OK) {
					id = null;
					op = sync_result_bool (true);
				} else {
					op = sync_error (new Backend.Error.INTERNAL_ERROR (db.errmsg()));
				}
			}

			return op;
		}
	}
}
