/* sql-query-visitor.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People;
using People.Utils;


namespace People.Backend.Sqlite {
	/**
	 * SQLQueryVisitor is an implementation of {@link QueryVisitor} translating
	 * {@link Query} trees into SQL queries.
	 */
	public class SQLQueryVisitor : Object, Backend.QueryVisitor {
		private static ReadOnlyMap<string, string> ns_to_col = null;

		private StringBuilder sql_query;

		construct {
			var ns_to_col = new HashMap<string, string> (str_hash, str_equal, str_equal);
			ns_to_col.set ("identity.name.first", "first_name");
			ns_to_col.set ("identity.name.middle", "middle_name");
			ns_to_col.set ("identity.name.last", "last_name");
			ns_to_col.set ("communication.phone.home", "home_phone");
			ns_to_col.set ("communication.phone.work", "work_phone");
			ns_to_col.set ("communication.phone.mobile", "mobile_phone");
			ns_to_col.set ("communication.email.home", "home_email");
			ns_to_col.set ("communication.email.work", "work_email");
			this.ns_to_col = new ReadOnlyMap<string, string> (ns_to_col);
		}

		/**
		 * Translates a {@link Query} into a SQL query.
		 *
		 * @param query the Query to translate. If <code>null</code> is passed,
		 *              then the method will return a SQL query selectiong all
		 *              the entries
		 * @return      a SQL query
		 */
		public string translate (Backend.Query? query) {
			sql_query = new StringBuilder ();
			sql_query.append ("SELECT id FROM people");
			if (query != null) {
				sql_query.append(" WHERE ");
				query.accept (this);
			}
			sql_query.append (";");
			return sql_query.str;
		}

		public void visit_composite (Backend.CompositeQuery query) {
			sql_query.append_printf ("(");

			if (query.operator == Backend.QueryOperator.NOT) {
				sql_query.append_printf("%s ", label (query.operator));
				query.queries.get (0).accept (this);
			} else {
				var first = true;
				foreach (Query q in query.queries) {
					if (first) {
						first = false;
					} else {
						sql_query.append_printf (" %s ", label (query.operator));
					}
					q.accept (this);
				}
			}

			sql_query.append_printf (")");
		}

		public void visit_field (FieldQuery query) {
			var column = ns_to_col.get (query.field);
			if (column == null) {
				column = query.field;
			}

			sql_query.append_printf ("%s", column);
		}

		public void visit_value (ValueQuery query) {
			sql_query.append_printf ("\"%s\"", query.value);
		}

		private static string label (Backend.QueryOperator operator) {
			switch (operator) {
				case Backend.QueryOperator.AND:
					return "AND";
				case Backend.QueryOperator.NOT:
					return "NOT";
				case Backend.QueryOperator.OR:
					return "OR";
				case Backend.QueryOperator.EQUALS:
					return "LIKE";
				case Backend.QueryOperator.CONTAINS:
					return "LIKE";
			}
			assert_not_reached ();
		}
	}



}
