/* contact.vala
 *
 * Copyright (C) 2008 Jiqing Qiang <jiqing.qiang@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;
using Soup;
using Json;

using People;
using People.Utils;


namespace People.Backend.Twitter {

	public class Contact : Backend.BaseContact {
		private Map<Soup.Message, AsyncOperation> pending_operations;
		private Map<Soup.Message, Gee.Set<string> > pending_requests_fields;

		public Soup.Session session {
			private get;
			construct;
		}

		private Backend.ContactFieldSpec default_field_spec =
				new ContactFieldSpec(Backend.ContactFieldFlags.WRITABLE);

		construct {
			pending_operations = new HashMap<Soup.Message, AsyncOperation> ();
			pending_requests_fields = new HashMap<Soup.Message, Gee.Set<string> > ();

			flags = Backend.ContactFlags.NONE;
		}

		public Contact (string id, Soup.Session session) {
			this.id = id;
			this.session = session;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields () {
			var operation = async_operation_init ();

			var url = "http://twitter.com/users/show/%s.json".printf(id);
			var msg = new Soup.Message ("GET", url);
			pending_operations.set (msg, operation);
			session.queue_message (msg, get_defined_fields_callback);
			return operation;
		}

		private void get_defined_fields_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);

			if (msg.status_code != KnownStatusCode.OK) {
				async_operation_emit (operation, new Gee.HashSet<string> (str_hash, str_equal));
				pending_operations.remove (msg);
				return;
			}

			var parser = new Parser ();
			parser.load_from_data (msg.response_body.data,
						(ulong) msg.response_body.data.size);
			var fields = extract_fields (parser).get_keys ();
			async_operation_emit (operation, fields);
			pending_operations.remove (msg);
		}

		public override AsyncOperation<Gee.Map<string, string> >
				get_fields (Gee.Set<string> field_names) {
			var operation = async_operation_init ();

			var url = "http://twitter.com/users/show/%s.json".printf(id);
			var msg = new Soup.Message ("GET", url);

			pending_operations.set (msg, operation);

			session.queue_message (msg, get_fields_callback);
			return operation;
		}

		private void get_fields_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);

			if (msg.status_code != KnownStatusCode.OK) {
				async_operation_emit (operation, new Gee.HashMap<string, string> (str_hash, str_equal, str_equal));
				pending_operations.remove (msg);
				return;
			}
			var parser = new Parser ();
			parser.load_from_data (msg.response_body.data,
						(ulong) msg.response_body.data.size);

			var values = extract_fields (parser);

			async_operation_emit (operation, values);
			pending_operations.remove (msg);
			pending_requests_fields.remove (msg);
		}

		public override AsyncOperation<ContactFieldSpec> get_field_spec (string field_name) {
			return sync_result (default_field_spec);
		}

		public override AsyncOperation<Gee.Set<string> > get_definable_fields () {
			return sync_result (new Gee.HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields () {
			return sync_result (new Gee.HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<Gee.Set<string> > set_fields (Gee.Map<string, string> fields) {
			return sync_result (new Gee.HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<bool> @delete () {
			return sync_result_bool (false);
		}


		private Gee.Map<string, string> extract_fields (Parser parser) {
			weak Json.Object root = parser.get_root ().get_object ();

			var fields = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);

			weak Json.Node node;

			/* full name */
			node = root.get_member ("name");
			if (node != null) {
				var full = node.get_string ();
				fields.set ("identity.name.full", full);
			}

			/* screen name */
			node = root.get_member ("screen_name");
			if (node != null) {
				var screen = node.get_string ();
				fields.set ("identity.name.nickname", screen);
			}

			/* location */
			node = root.get_member ("location");
			if (node != null) {
				var location = node.get_string ();
				fields.set ("identity.location", location);
			}

			/* description */
			node = root.get_member ("description");
			if (node != null) {
				var description = node.get_string ();
				fields.set ("identity.description", description);
			}

			/* profile image url */
			node = root.get_member ("profile_image_url");
			if (node != null) {
				var avatar = node.get_string ();
				fields.set ("communication.web.twitter.avatar", avatar);
			}

			/* personal website */
			node = root.get_member ("url");
			if (node != null) {
				var url = node.get_string ();
				fields.set ("communication.web.url", url);
			}

			return fields;
		}

	}

}
