/* test-contact-source.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */


using People;
using Core.Async;


namespace People.Backend.Test {

	public abstract class ContactSource : Core.Trial.Suite {
		protected string backend_name = "Null";
		protected Backend.ContactSourceFlags contact_source_flags;
		protected uint contacts_count = 0;
		protected string valid_contact_handle = "foo-bar";
		protected string invalid_contact_handle = "abcd-wzyx";

		private uint contacts_current_count = 0;
		private uint contacts_target_count = 0;

		protected Backend.ContactSource contact_source;

		protected override void initialize () {
			name = "People::Backend::%s::ContactSource".printf (backend_name);
			add_test_case ("flags", test_flags);
			add_test_case ("initial contacts count", test_initial_contacts_count);
			add_test_case ("request_contact with valid handle",
					test_request_contact_valid);
			add_test_case ("request_contact with invalid handle",
					test_request_contact_invalid);
			add_test_case ("request_contact with null handle",
					test_request_contact_null);
			add_test_case ("contact creation",
					test_contact_creation);
		}

		protected override void set_up () {
		}

		protected override void tear_down () {
			contact_source = null;
		}

		private void async_callback (AsyncResult async_result) {
			async_break ();
		}

		private void test_flags () {
			assert_true (contact_source_flags == contact_source.flags);
		}

		private void test_request_contact_valid () {
			try {
				var async_result = contact_source.request_contact (valid_contact_handle);
				async_result.add_callback (async_callback);
				async_wait ();

				var contact = async_result.get();
				assert_true (contact != null);
			} catch (Error e) {
				failure ();
			}
		}

		private void test_request_contact_invalid () {
			try {
				var async_result = contact_source.request_contact (invalid_contact_handle);
				async_result.add_callback (async_callback);
				async_wait ();

				async_result.get();
				failure ();
			} catch (Error e) {
				success ();
			}
		}

		private void test_request_contact_null () {
			if (Backend.ContactSourceFlags.CAN_CREATE_CONTACT in contact_source.flags) {
				try {
					var async_result = contact_source.request_contact (null);
					async_result.add_callback (async_callback);
					async_wait ();

					var contact = async_result.get();
					assert_true (contact != null);
				} catch (Error e) {
					failure ();
				}
			} else {
				try {
					var async_result = contact_source.request_contact (null);
					async_result.add_callback (async_callback);
					async_wait ();

					async_result.get();
					failure ();
				} catch (Error e) {
					success ();
				}
			}
		}

		private void test_contact_creation () {
			if (! (Backend.ContactSourceFlags.CAN_CREATE_CONTACT in contact_source.flags))
				return;

			Backend.Contact contact = null;
			try {
				var async_result = contact_source.request_contact (null);
				async_result.add_callback (async_callback);
				async_wait ();

				contact = async_result.get();
			} catch (Error e) {
				failure ();
			}

			var fields = new Core.Container.HashMap<string, string>(str_hash, str_equal, str_equal);
			fields.set ("identity.name.first", "hello");
			fields.set ("identity.name.last", "world");
			contact.set_fields (fields);

			assert_true (contact.id != null);
			check_contacts_count (contacts_count + 1);
		}

		private void test_initial_contacts_count () {
			check_contacts_count (contacts_count);
		}

		private void check_contacts_count(uint count) {
			contacts_target_count = count;
			var search = ((Backend.DumpableContactSource)contact_source).dump ();
			try {
				var contacts = search.get ();
				contacts.added += (contacts, contact) => {
					contacts_current_count++;
				};
				search.notify["complete"] += src => {
					assert_true (contacts_current_count == contacts_target_count);
					contacts_current_count = 0;
					async_break ();
				};
				async_wait ();
			} catch {
				failure ();
			}
		}

	}
}
