/* test-contact.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */


using People;
using Core.Async;
using Core.Container;


namespace People.Backend.Test {

	public abstract class Contact : Core.Trial.Suite {
		protected string backend_name = "Null";
		protected string contact_id = null;
		protected Backend.ContactFlags contact_flags;
		protected Set<string> mandatory_fields;
		protected Set<string> defined_fields;
		protected Set<string> definable_fields;

		protected People.Backend.Contact contact;
		private Set<string> fields_reference;

		protected override void initialize () {
			name = "People::Backend::%s::Contact".printf (backend_name);
			add_test_case ("contact ID", test_id);
			add_test_case ("flags", test_flags);
			add_test_case ("mandatory fields", test_mandatory_fields);
			add_test_case ("definable fields", test_definable_fields);
			add_test_case ("defined fields", test_defined_fields);
			add_test_case ("set fields", test_set_fields);
			add_test_case ("get fields", test_get_fields);

			mandatory_fields = new Core.Container.HashSet<string> (str_hash, str_equal);
			definable_fields = new Core.Container.HashSet<string> (str_hash, str_equal);
			defined_fields = new Core.Container.HashSet<string> (str_hash, str_equal);
		}

		protected override void set_up () {
		}

		protected override void tear_down () {
			contact = null;
			mandatory_fields.clear ();
			definable_fields.clear ();
			defined_fields.clear ();
		}

		private void test_flags () {
			assert_true (contact_flags == contact.flags);
		}

		private void test_id () {
			assert_true (contact_id == contact.id);
		}

		private void test_mandatory_fields () {
			fields_reference = mandatory_fields;
			contact.get_mandatory_fields().add_callback(fields_callback);
			async_wait ();
		}

		private void test_defined_fields () {
			fields_reference = defined_fields;
			contact.get_defined_fields().add_callback(fields_callback);
			async_wait ();
		}

		private void test_definable_fields () {
			fields_reference = definable_fields;
			contact.get_definable_fields().add_callback(fields_callback);
			async_wait ();
		}

		private void test_set_fields () {
			if (! (Backend.ContactFlags.EDITABLE in contact_flags)) {
				return;
			}

			AsyncResultCallback callback = (op) => {
				Core.Container.Set<string> fields;
				try {
					fields = (Core.Container.Set<string>) op.get();
					assert_true (fields != null);
				} catch (Error e) {
					async_failure (e.message);
					return;
				}
				assert_true(fields.size == 2);
				assert_true(fields.contains("identity.name.first"));
				assert_true(fields.contains("identity.name.last"));
				async_break ();
			};

			var fields = new Core.Container.HashMap<string, string>(str_hash, str_equal, str_equal);
			fields.set ("identity.name.first", "hello");
			fields.set ("identity.name.last", "world");

			contact.set_fields(fields).add_callback(callback);
			async_wait ();
		}

		private void test_get_fields () {
			AsyncResultCallback callback = (op) => {
				Core.Container.Map<string> fields;
				try {
					fields = (Core.Container.Map<string>) op.get();
					assert_true (fields != null);
				} catch (Error e) {
					async_failure (e.message);
					return;
				}
				assert_true(fields.size == defined_fields.size);
				//assert_true(fields.contains("identity.name.first"));
				//assert_true(fields.contains("identity.name.last"));
				async_break ();
			};
			contact.get_fields(defined_fields).add_callback(callback);
			async_wait ();
		}

		private void fields_callback (AsyncResult<Core.Container.Set<string>> op) {
			Core.Container.Set<string> fields;
			try {
				fields = (Core.Container.Set<string>) op.get();
				assert_true (fields != null);
			} catch (Error e) {
				async_failure (e.message);
				return;
			}

			assert_true (fields.size == fields_reference.size);

			foreach (string field in fields_reference) {
				assert_true (fields.contains (field));
			}

			foreach (string field in fields) {
				assert_true (fields_reference.contains (field));
			}
			async_break ();
		}

	}
}
