/* base-contact-source-factory.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Core.Container;
using Core.Async;

namespace People.Backend {
	/**
	 * BaseContactSourceFactory is an abstract class implementing
	 * {@link ContactSourceFactory}.
	 * <p>
	 * This class implements parameter checking for the
	 * {@link ContactSourceFactory#request_contact_source()} method
	 * according to the {@link ContactSourceFactory#get_parameters()} result.
	 */
	public abstract class BaseContactSourceFactory : Object, ContactSourceFactory {

		/* ContactSourceFactory interface methods */
		public abstract ContactSourceParamSpec[] get_parameters ();

		public AsyncResult<ContactSource> request_contact_source (Map<string, string>? parameters) {
			ContactSourceParamSpec[] specifications = get_parameters ();

			foreach (ContactSourceParamSpec spec in specifications) {
				if (ContactSourceParamFlags.MANDATORY in spec.flags) {
					if (parameters == null || !parameters.contains (spec.name)) {
						var error = new Error.INVALID_ARGUMENT ("Missing mandatory parameter:" + spec.name);
						return new ImmediateResult<ContactSource>.with_error (error);
					}
				}
			}

			return get_contact_source (parameters);
		}
		/* end of ContactSourceFactory interface methods */

		/**
		 * This method is called by the base class when the parameters
		 * validation was successful. It must be implemented by the inheriting
		 * class
		 *
		 * @return a #ContactSource instance
		 */
		protected abstract AsyncResult<ContactSource> get_contact_source (Map<string, string>? parameters);
	}
}
