/* base-contact-source.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Core.Container;
using Core.Async;

namespace People.Backend {
	/**
	 * BaseContactSource is an abstract class implementing the
	 * {@link ContactSource} interface.
	 * <p>
	 * The user of this class MUST implement {@link BaseContactSource.validate_contact_id}
	 * and {@link BaseContactSource.create_contact_object}.
	 */
	public abstract class BaseContactSource : Object, ContactSource {

		private ContactManager contact_manager = new ContactManager ();
		private ResultManager result_manager = new ResultManager ();
		private Map<AsyncResult<bool>, string> validation_map = new HashMap<AsyncResult<bool>, string> ();

		/* ContactSource interface */
		public ContactSourceFlags flags {
			get;
			protected set;
			default = ContactSourceFlags.NONE;
		}

		public AsyncResult<Contact> request_contact (string? id) {
			if (id == null && !(ContactSourceFlags.CAN_CREATE_CONTACT in flags)) {
				var error = new Error.NOT_SUPPORTED ("Contact creation not supported by this backend");
				return new ImmediateResult<Contact>.with_error (error);
			}

			if (id == null) {
				return new ImmediateResult<Contact> (get_contact_object (null));
			} else {
				var validation = validate_contact_id (id);
				validation_map.set (validation, id);
				var result = new ResultChain<Contact> (validation);
				result.add_transform ((async_result) => {
					try {
						bool valid_id = (bool) async_result.get ();
						string contact_id = validation_map.get (async_result);
						validation_map.remove (async_result);
						if (valid_id)
							return new ImmediateResult<Contact> (get_contact_object (contact_id));
						else
							return new ImmediateResult<Contact>.with_error (new Error.INVALID_ARGUMENT ("Requested a contact with an unknown ID: " + contact_id));
					} catch (Error e) {
						return new ImmediateResult<Contact>.with_error (e);
					}
				});
				result_manager.add (result);
				return result;
			}
		}
		/* end of ContactSource interface */

		protected Contact get_contact_object (string? id) {
			Contact contact = null;

			if (id != null) {
				contact = contact_manager.request_contact (id);
			}

			if (contact == null) {
				contact = create_contact_object (id);
				register_contact (contact);
			}

			return contact;
		}

		protected void register_contact (Contact contact) {
			if (contact_manager.register_contact (contact)) {
				contact.notify["id"] += contact_state_changed;
			}
		}

		/**
		 * The user of this class MUST implement this method to allow the validation
		 * and checking for the existence of the contact ID.
		 */
		protected abstract AsyncResult<bool> validate_contact_id (string id);

		/**
		 * The user of this class MUST implement this method returning a Contact object
		 * corresponding to the contact ID. The implementation MUST NOT check for the validity
		 * of the given ID.
		 */
		protected abstract Contact create_contact_object (string? id);

		private void contact_state_changed (Contact contact, ParamSpec pspec) {
			if (contact.id == null)
				contact_removed (contact);
			else
				contact_added (contact);
		}
	}
}
