/* contact-manager.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Core.Container;

namespace People.Backend {
	/**
	 * A ContactManager is an entity used to manage {@link Contact} objects
	 * lifecycle in a {@link ContactSource}.
	 */
	internal class ContactManager : Object {
		protected Set<weak Contact> contacts;

		construct {
			contacts = new HashSet<weak Contact> ();
		}

		~ContactManager () {
			foreach (Contact contact in contacts) {
				contact.weak_unref (end_of_use, this);
			}
			contacts.clear ();
		}

		/**
		 * Register a {@link Contact} with the ContactManager.
		 *
		 * @param contact the contact to register
		 */
		public bool register_contact (Contact contact) {
			if (! (contact in contacts)) {
				contact.weak_ref (end_of_use, this);
				contact.notify["id"] += contact_state_changed;
				contacts.add (contact);
				return true;
			}
			return false;
		}

		/**
		 * Requests a {@link Contact} instance with the given id.
		 *
		 * @param id the identifier for the wanted Contact.
		 * @return   an instance of Contact or null if the contact is not
		 *           existant
		 */
		public Contact? request_contact (string id) {
			//TODO: optimize the data structure to allow immediate access
			foreach (Contact contact in contacts) {
				if (contact.id == id) {
					return contact;
				}
			}
			return null;
		}

		private void end_of_use (Object object) {
			var contact = (Contact) object;
			contacts.remove (contact);
		}

		private void contact_state_changed (Contact contact, ParamSpec pspec) {
			if (contact.id == null) {
				contact.weak_unref (end_of_use, this);
				contacts.remove (contact);
			}
		}
	}
	}

}
