/* contact-source-factory.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Core.Container;
using Core.Async;

namespace People.Backend {

	[Flags]
	public enum ContactSourceParamFlags {
		NONE      = 0,
		MANDATORY = 1 << 0,
		HIDDEN    = 1 << 1
	}

	/**
	 * An ContactSourceParamSpec object contains a description for a parameter used
	 * to request an {@link ContactSource} from the {@link ContactSourceFactory}.
	 */
	public class ContactSourceParamSpec : Object {
		/**
		 * Parameter name.
		 * <p>
		 * It must be unique for a given {@link ContactSourceFactory}.
		 */
		public string name {
			get;
			construct;
		}

		/**
		 * Parameter type.
		 */
		public Type gtype {
			get;
			construct;
		}

		/**
		 * A combination of {@link ContactSourceParamFlags}.
		 */
		public ContactSourceParamFlags flags {
			get;
			construct;
		}

		/**
		 * ContactSourceParamSpec constructor.
		 *
		 * @param name  parameter name
		 * @param gtype parameter type
		 * @param flafs combination of {@link ContactSourceParamFlags}
		 */
		public ContactSourceParamSpec(string name, Type gtype, ContactSourceParamFlags flags) {
			this.name = name;
			this.gtype = gtype;
			this.flags = flags;
		}
	}

	/**
	 * This interface defines methods to be implemented by factories to provide
	 * {@link ContactSource} instances through the request_contact_source
	 * method.
	 * <p>
	 * The interface provides a description of the possible and mandatory
	 * parameters for requesting an ContactSource through the get_parameters
	 * method.
	 */
	public interface ContactSourceFactory : Object {
		/**
		 * Returns an array of {@link ContactSourceParamSpec} describing the
		 * parameters needed to require a new {@link ContactSource} from this
		 * factory.
		 * <p>
		 * The sort of parameters passed here include credentials, connection
		 * parameters, authentication parameters, etc.
		 *
		 * @return an array of ContactSourceParamSpec
		 */
		public abstract ContactSourceParamSpec[] get_parameters ();

		/**
		 * Returns an {@link ContactSource} from the given parameters.
		 *
		 * @param parameters a {@link Map} associating a parameter name to its
		 *                   value
		 * @return           an ContactSource instance
		 * @throws           {@link Error#NOT_AVAILABLE} if the ContactSource
		 *                   is temporarly unavailable
		 * @throws           {@link Error#INVALID_ARGUMENT} if the parameter
		 *                   constraints determined for the ContactSource were
		 *                   not met or if the creation of the ContactSource
		 *                   failed
		 * @throws           {@link Error#NOT_AUTHORIZED} if the ContactSource
		 *                   cannot be accessed with the provided credentials
		 * @throws           {@link Error#NOT_AUTHENTICATED} if the authentication
		 *                   using the provided credentials fails
		 * @throws           {@link Error#NETWORK_REQUIRED} if the operation requires
		 *                   a network access that is not available
		 */
		// FIXME: fix this when Vala supports generic values in a better way
		public abstract AsyncResult<ContactSource> request_contact_source (Map<string, string>? parameters);
	}

}
