/* query.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Core.Container;
using Core.Async;

namespace People.Backend {
	public enum QueryOperator {
		NOT,
		AND,
		OR,
		EQUALS,
		CONTAINS,
	}

	public errordomain QueryError {
		UNEXPECTED_STRUCTURE
	}

	public interface QueryVisitor : Object {
		public abstract void visit_composite (CompositeQuery query);
		public abstract void visit_field (FieldQuery query);
		public abstract void visit_value (ValueQuery query);
	}

	public abstract class Query : Object {
		public abstract void accept (QueryVisitor visitor);

		public static Query not (Query query) {
			var queries = new ArrayList<Query> ();
			queries.add (query);
			return new CompositeQuery (QueryOperator.NOT, queries);
		}

		public static Query and (Core.Container.List<Query> queries) {
			return compose (QueryOperator.AND, queries);
		}

		public static Query or (Core.Container.List<Query> queries) {
			return compose (QueryOperator.OR, queries);
		}

		private static Query compose (QueryOperator op, Core.Container.List<Query> queries) {
			return new CompositeQuery (op, queries);
		}
	}

	public class CompositeQuery : Query {
		public QueryOperator operator {
			public get;
			private construct set;
		}

		public Core.Container.List<Query> queries {
			public get;
			private construct set;
		}

		public CompositeQuery (QueryOperator operator, Core.Container.List<Query> queries) {
			this.operator = operator;
			this.queries = queries;
		}

		public override void accept (QueryVisitor visitor) {
			visitor.visit_composite (this);
		}

		public bool evaluate (Map<string, string> fields) throws QueryError {
			switch (operator) {
				case QueryOperator.NOT:
					var subquery = queries.get (0);
					if (! (subquery is CompositeQuery)) {
						throw new QueryError.UNEXPECTED_STRUCTURE ("Expected composite query");
					}
					return !(((CompositeQuery) subquery).evaluate (fields));
				case QueryOperator.AND:
					var value = true;
					for (int i = 0; (i < queries.size) && value; i++) {
						var subquery = queries.get (i);
						if (! (subquery is CompositeQuery)) {
							throw new QueryError.UNEXPECTED_STRUCTURE ("Expected composite query");
						}
						value &= ((CompositeQuery) subquery).evaluate (fields);
					}
					return value;
				case QueryOperator.OR:
					var value = false;
					for (int i = 0; (i < queries.size) && !value; i++) {
						var subquery = queries.get (i);
						if (! (subquery is CompositeQuery)) {
							throw new QueryError.UNEXPECTED_STRUCTURE ("Expected composite query");
						}
						value |= ((CompositeQuery) subquery).evaluate (fields);
					}
					return value;
				case QueryOperator.CONTAINS:
					string lvalue = get_value (queries.get (0), fields);
					string rvalue = get_value (queries.get (1), fields);

					if ((lvalue == null) || (rvalue == null)) {
						/* needed fields are not available */
						return false;
					}
					return (lvalue.str (rvalue) != null);
				case QueryOperator.EQUALS:
					string lvalue = get_value (queries.get (0), fields);
					string rvalue = get_value (queries.get (1), fields);

					if ((lvalue == null) || (rvalue == null)) {
						/* needed fields are not available */
						return false;
					}
					return (lvalue == rvalue);
			}
			return false;
		}

		private string? get_value (Query query, Map<string, string> fields)
				throws QueryError {
			if (query is FieldQuery) {
				return fields.get (((FieldQuery) query).field);
			} else if (query is ValueQuery) {
				return ((ValueQuery) query).value;
			}
			throw new QueryError.UNEXPECTED_STRUCTURE ("Unexpected composite query");
		}
	}

	public class FieldQuery : Query {
		public string field {
			public get;
			private construct set;
		}

		public FieldQuery (string field) {
			this.field = field;
		}

		public override void accept (QueryVisitor visitor) {
			visitor.visit_field (this);
		}
	}

	// TODO: use a Value instead of a string
	public class ValueQuery : Query {
		public string value {
			public get;
			private construct set;
		}

		public ValueQuery (string value) {
			this.value = value;
		}

		public override void accept (QueryVisitor visitor) {
			visitor.visit_value (this);
		}
	}
}
