/* contact-manager.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People.Utils;


namespace People {
	public class ContactManager : AsyncOperationHelper {
		public AddressBook address_book {
			private get;
			private construct;
		}

		public Set<Gee.List<string>> merged_contacts {
			private get;
			private construct;
		}

		private Map<string, weak People.Contact> contacts;

		private Map<string, AsyncOperation<People.Contact>> handled;
		private Map<MultiAsyncOperation<Backend.Contact>, string> builders;
		private Map<MultiAsyncOperation<Backend.Contact>, AsyncOperation<People.Contact>> pending;

		construct {
			contacts = new HashMap<string, weak People.Contact> (str_hash, str_equal);

			handled = new HashMap<string, AsyncOperation<People.Contact>> (str_hash, str_equal);
			builders = new HashMap<Backend.Contact, string> (direct_hash, direct_equal, str_equal);
			pending = new HashMap<MultiAsyncOperation<Backend.Contact>, AsyncOperation<People.Contact>> ();
		}

		public ContactManager (AddressBook address_book,
				Set<Gee.List<string>> merged_contacts) {
			this.address_book = address_book;
			this.merged_contacts = merged_contacts;
		}

		~ContactManager () {
			foreach (string id in contacts.get_keys ()) {
				contacts.get (id).weak_unref (end_of_use, this);
			}
			contacts.clear ();
		}

		private void end_of_use (Object object) {
			var contact = (People.Contact) object;

			foreach (string id in contacts.get_keys ()) {
				if (contacts.get (id) == contact) {
					contacts.remove (id);
					break;
				}
			}
		}

		private string get_all_full_contact_id (string full_contact_id) {
			foreach (Gee.List<string> ids in merged_contacts) {
				if (full_contact_id in ids) {
					string all_full_contact_id = ids.get (0);
					for (int i = 1; i < ids.size; i++) {
						all_full_contact_id += "+" + ids.get (i);
					}
					return all_full_contact_id;
				}
			}
			return full_contact_id;
		}

		public AsyncOperation<People.Contact> request_contact (Backend.ContactSource source,
				Backend.Contact contact) {
			string source_id = address_book.sources.get_x (source);
			string full_contact_id = "%s:%s".printf (source_id, contact.id);
			string all_full_contact_id = get_all_full_contact_id (full_contact_id);

			if (all_full_contact_id in contacts.get_keys ()) {
				/* the corresponding People.Contact already exists*/
				var result = contacts.get (all_full_contact_id);

				if (result is People.MultiContact) {
					foreach (Gee.List<string> ids in merged_contacts) {
						var index = ids.index_of (full_contact_id);
						if (index >= 0) {
							People.MultiContact m = (People.MultiContact) result;
							Backend.Contact c = m.contacts.get (index);
							if (c == null) {
								m.contacts.set (index, contact);
							}
							break;
						}
					}
				}

				return sync_result (result);
			}

			var pending_operation = handled.get (all_full_contact_id);
			if (pending_operation != null) {
				/* the corresponding People.Contact is already being built */
				return pending_operation;
			}

			/* we need to build the corresponding People.Contact */
			Gee.List<string> all_full_contact_ids = null;
			foreach (Gee.List<string> full_contact_ids in merged_contacts) {
				foreach (string id in full_contact_ids) {
					if (id == full_contact_id) {
						/* other Backend.Contact in that People.Contact */
						all_full_contact_ids = full_contact_ids;
						break;
					}
				}
			}

			var op = async_operation_init ();
			handled.set (all_full_contact_id, op);
			build_contact (op, all_full_contact_ids, source_id, contact);
			return op;
		}

		private void build_contact (AsyncOperation<People.Contact> operation,
				Gee.List<string>? all_full_contact_ids,
				string source_id,
				Backend.Contact contact) {
			if (all_full_contact_ids != null) {
				/* get all the Backend.Contact needed to build the People.MultiContact */
				var ops = new ArrayList<AsyncOperation<Backend.Contact> > ();

				string all_full_contact_id = "";
				for (int i = 0; i < all_full_contact_ids.size; i++) {
					string full_contact_id = all_full_contact_ids.get (i);

					all_full_contact_id += full_contact_id;
					if (i < (all_full_contact_ids.size - 1)) {
						all_full_contact_id += "+";
					}

					var ids = full_contact_id.split (":");
					AsyncOperation<Backend.Contact> op;
					if ((ids[0] == contact.id) && (ids[1] == source_id)) {
						/* no need to fetch this Backend.Contact again */
						op = new AsyncOperation<Backend.Contact> ();
						op.completed_with_result (contact);
					} else {
						op = address_book.sources.get_y (ids[0]).request_contact (ids[1]);
					}
					ops.add (op);
				}

				var builder = new MultiAsyncOperation<Backend.Contact> (ops);
				pending.set (builder, operation);
				builders.set (builder, all_full_contact_id);
				builder.add_callback (builder_callback);
			} else {
				/* we already have the Backend.Contact to build the People.SingleContact */
				var full_contact_id = "%s:%s".printf (source_id, contact.id);

				debug ("Creating single contact: %s", full_contact_id);
				var c = new People.SingleContact (full_contact_id, contact);
				contacts.set (full_contact_id, c);
				c.weak_ref (end_of_use, this);

				async_operation_emit (operation, c);
				handled.remove (full_contact_id);
			}
		}

		private void builder_callback (MultiAsyncOperation<Backend.Contact> builder, void* data) {
			string all_full_contact_id = builders.get (builder);
			try {
				var c = builder.get_result ();
				debug ("Creating multi contact: %s", all_full_contact_id);
				var contact = new People.MultiContact (all_full_contact_id, c);
				contacts.set (all_full_contact_id, contact);
				contact.weak_ref (end_of_use, this);

				async_operation_emit (pending.get (builder), contact);
				handled.remove (all_full_contact_id);
			} catch (Error e) {
				warning ("builder_callback: %s", e.message);
			}

			builders.remove (builder);
			pending.remove (builder);
		}
	}

}
