/* proxy-contact-source.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People.Utils;

namespace People {

	public class ProxyContactSource : AsyncOperationHelper, Backend.ContactSource,
			Backend.DumpableContactSource, Backend.QueryableContactSource {
		private Gee.Map<AsyncSearch<Backend.Contact>,
					AsyncSearch<Backend.Contact> > pending_searches;

		private QueryableContactSourceHelper queryable = null;

		public Backend.ContactSource source {
			private get;
			private construct set;
		}

		public Backend.ContactSourceFlags flags {
			get { return source.flags; }
			private set { source.flags = flags; }
		}

		public bool proxify_contacts {
			private get;
			private construct set;
		}

		construct {
			pending_searches = new Gee.HashMap<AsyncSearch<Backend.Contact>,
						AsyncSearch<Backend.Contact>> ();

			source.contact_added += contact_added_hook;
			source.contact_removed += contact_removed_hook;
		}

		/**
		 * ProxyContactSource constructor.
		 *
		 * @param source the {@link ContactSource} to be proxified
		 */
		public ProxyContactSource (Backend.ContactSource source,
				bool proxify_contacts = false) {
			this.source = source;
			this.proxify_contacts = proxify_contacts;
		}

		private void contact_added_hook (Backend.ContactSource source,
				Backend.Contact contact) {
			contact_added (contact);
		}

		private void contact_removed_hook (Backend.ContactSource source,
				Backend.Contact contact) {
			contact_removed (contact);
		}

		/* ContactSource interface */
		public AsyncOperation<Backend.Contact> request_contact (string? id = null) {
			var op = async_operation_init ();

			var request = source.request_contact (id);
			pending_operations.add (request);
			request.add_callback (request_contact_callback, op);

			return op;
		}

		private void request_contact_callback(AsyncOperation<Backend.Contact> request,
				void* data) {
			var op = (AsyncOperation<Backend.Contact>) data;

			try {
				var contact = (Backend.Contact) request.get_result ();
				if (proxify_contacts) {
					op.completed_with_result (new ProxyContact (contact));
				} else {
					op.completed_with_result (contact);
				}
			} catch (Error e) {
				warning ("%s", e.message);
				op.completed_with_error (e);
			}

			pending_operations.remove (request);
		}
		/* end of ContactSource interface */

		/* DumpableContactSource interface */
		public AsyncSearch<Contact> dump () {
			var search = ((Backend.DumpableContactSource) source).dump ();
			return proxify (search);
		}
		/* end of DumpableContactSource interface */

		/* QueryableContactSource interface */
		public AsyncSearch<Contact> query (Backend.Query? query) {
			if (source is Backend.QueryableContactSource) {
				var search = ((Backend.QueryableContactSource) source).query (query);
				return proxify (search);
			} else if (queryable == null) {
				queryable = new QueryableContactSourceHelper (this);
			}
			return proxify (queryable.query (query));
		}
		/* end of QueryableContactSource interface */

		private AsyncSearch<Contact> proxify (AsyncSearch<Contact> search) {
			if (!proxify_contacts) {
				return search;
			}

			var proxy_search = new AsyncSearch<Backend.Contact> ();
			pending_searches.set (search, proxy_search);

			search.hit_found += hit_found_callback;
			search.done += done_callback;
			search.start ();

			return proxy_search;
		}

		private void hit_found_callback(AsyncSearch<Contact> search, Contact hit) {
			var proxy_search = pending_searches.get (search);

			var contact = hit as Backend.Contact;
			if (! (contact is ProxyContact)) {
				contact = new ProxyContact (contact);
			}

			proxy_search.add_hit (contact);
		}

		private void done_callback(AsyncSearch<Backend.Contact> search) {
			pending_searches.get (search).finish ();
			pending_searches.remove (search);
		}
	}
}
