/* query-evaluator.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People.Utils;


namespace People {
	/**
	 * QueryEvaluator is a class to be used to evaluate a single {@link Query}
	 * on several {@link Contact} objects.
	 */
	public class QueryEvaluator : AsyncOperationHelper, Backend.QueryVisitor {
		private Set<string> required_fields;

		public Backend.Query query {
			private get;
			private construct set;
		}

		construct {
			required_fields = new HashSet<string> (str_hash, str_equal);
			extract_required_fields ();
		}

		private void extract_required_fields () {
			query.accept (this);
		}

		/**
		 * QueryEvaluationHelper constructor.
		 *
		 * @param query the {@link Query} to evaluate
		 */
		public QueryEvaluator (Backend.Query query) {
			this.query = query;
		}

		/**
		 * Evaluates if the given {@link Contact} is selected by the
		 * {@link Query} of this QueryEvaluator.
		 *
		 * @param contact the Contact to be evaluated
		 * @return        a boolean {@link AsyncOperation} to be completed with
		 *                <code>true</code> if the Contact is selected by the
		 *                Query, <code>false</code> else
		 */
		public AsyncOperation<bool> evaluate (Backend.Contact contact) {
			if (query is Backend.CompositeQuery) {
				var op = async_operation_init_bool();

				var operation = contact.get_fields (required_fields);
				operation.add_callback (get_fields_callback, op);
				pending_operations.add (operation);

				return op;
			}
			return sync_result_bool (false);
		}

		private void get_fields_callback(AsyncOperation<Map<string, string> > operation,
				void* data) {
			var op = (AsyncOperation<bool>) data;

			var fields = (Map<string, string>) operation.get_result ();
			try {
				bool result = ((Backend.CompositeQuery) query).evaluate (fields);
				async_operation_emit_bool (op, result);
			} catch (Backend.QueryError e) {
				async_operation_error (op, e);
			}
			pending_operations.remove (operation);
		}

		/* QueryVisitor interface */
		public void visit_composite (Backend.CompositeQuery query) {
			foreach (Backend.Query subquery in query.queries) {
				subquery.accept (this);
			}
		}

		public void visit_field (Backend.FieldQuery query) {
			required_fields.add (query.field);
		}

		public void visit_value (Backend.ValueQuery query) {
		}
		/* end of QueryVisitor interface */
	}
}
