/* query-result-manager.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People.Utils;


namespace People {

	public class QueryResultManager : Object {
		private QueryBuilder query_builder;

		public Map<Backend.Query, weak QueryResult> queries;
		public Set<weak QueryResult> full;

		private Map<AsyncOperation<bool>, QueryResult> results;
		private Map<AsyncOperation<bool>, Backend.ContactSource> sources;

		construct {
			query_builder = new QueryBuilder ();

			queries = new HashMap<Backend.Query, weak QueryResult> ();
			full = new HashSet<weak QueryResult> ();

			results = new HashMap<AsyncOperation<bool>, QueryResult> ();
			sources = new HashMap<AsyncOperation<bool>, Backend.ContactSource> ();

			foreach (Backend.ContactSource source in address_book.sources.get_y_set ()) {
				source.contact_added += contact_added_callback;
				source.contact_removed += contact_removed_callback;
			}
		}

		public weak AddressBook address_book {
			private get;
			private construct;
		}

		public ContactManager contact_manager {
			private get;
			private construct;
		}

		public QueryResultManager (AddressBook address_book,
				ContactManager contact_manager) {
			this.address_book = address_book;
			this.contact_manager = contact_manager;
		}

		~QueryResultManager () {
			foreach (QueryResult result in full) {
				result.weak_unref (end_of_use, this);
			}
			full.clear ();
			foreach (Backend.Query query in queries.get_keys ()) {
				queries.get (query).weak_unref (end_of_use, this);
			}
			queries.clear ();
		}

		private void end_of_use (Object object) {
			var result = (QueryResult) object;

			if (!full.remove (result)) {
				foreach (Backend.Query query in queries.get_keys ()) {
					if (queries.get (query) == result) {
						queries.remove (query);
						break;
					}
				}
			}
		}

		private void contact_added_callback (Backend.ContactSource source,
				Backend.Contact contact) {
			foreach (QueryResult result in full) {
				result.include_contact (source, contact);
			}

			foreach (Backend.Query query in queries.get_keys ()) {
				var evaluator = new QueryEvaluator (query);
				var evaluation = evaluator.evaluate (contact);
				results.set (evaluation, queries.get (query));
				sources.set (evaluation, source);

				evaluation.add_callback (evaluate_callback, contact);
			}
		}

		private void evaluate_callback (AsyncOperation<bool> op, void* data) {
			var contact = (Backend.Contact) data;
			var query_result = results.get (op);
			var source = sources.get (op);

			bool result = op.get_result ();
			if (result) {
				query_result.include_contact (source, contact);
			}

			results.remove (op);
			sources.remove (op);
		}

		private void contact_removed_callback (Backend.ContactSource source,
				Backend.Contact contact) {
			foreach (QueryResult result in full) {
				result.exclude_contact (source, contact);
			}

			foreach (QueryResult result in queries.get_values ()) {
				result.exclude_contact (source, contact);
			}
		}

		public QueryResult query (string? query) {
			var searches = new Gee.HashMap<AsyncSearch, Backend.ContactSource> ();
			Backend.Query q = null;

			if (query == null) {
				foreach (Backend.ContactSource source in address_book.sources.get_y_set ()) {
					var search = ((Backend.DumpableContactSource) source).dump ();
					searches.set (search, source);
				}
			} else {
				q = query_builder.build (query);
				foreach (Backend.ContactSource source in address_book.sources.get_y_set ()) {
					var search = ((Backend.QueryableContactSource) source).query (q);
					searches.set (search, source);
				}

			}

			var search = new MultiAsyncSearch<Backend.ContactSource,
					Backend.Contact> (searches);
			var result = new QueryResult (contact_manager, search);

			result.weak_ref (end_of_use, this);
			if (q == null) {
				full.add (result);
			} else {
				queries.set (q, result);
			}

			return result;
		}
	}

}
