/* query-result.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People.Utils;


namespace People {

	public enum QueryResultState {
		IDLE,
		FETCHING,
		FETCHED
	}

	public class QueryResult : Object {
		public ContactManager contact_manager {
			private get;
			private construct;
		}

		public MultiAsyncSearch<Backend.ContactSource, Backend.Contact> search {
			private get;
			private construct set;
		}
		private bool finished;

		[Notify]
		public QueryResultState state {
			public get;
			private set;
		}

		public Gee.List<People.Contact> results;
		private Map<People.Contact, Set<Backend.Contact>> contents;
		public signal void result_fetched (Contact contact);
		public signal void result_discarded (Contact contact);

		private Set<AsyncOperation<People.Contact>> pending;

		construct {
			results = new ArrayList<People.Contact> ();
			contents = new HashMap<People.Contact, Set<Backend.Contact>> ();
			pending = new HashSet<People.Contact> ();

			search.hit_found += hit_found_callback;
			search.done += done_callback;

			finished = false;
			state = QueryResultState.IDLE;
		}

		public QueryResult (ContactManager contact_manager,
				MultiAsyncSearch<Backend.ContactSource, Backend.Contact> search) {
			this.contact_manager = contact_manager;
			this.search = search;
		}

		~QueryResult () {
			contents.clear ();
			results.clear ();
		}

		/* Public API */
		public void fetch () {
			state = QueryResultState.FETCHING;
			search.start ();
		}

		public Gee.Iterator get_iterator () {
			return results.iterator ();
		}
		/* End of public API */

		/* Internal API */
		public void include_contact (Backend.ContactSource source, Backend.Contact hit) {
			var op = contact_manager.request_contact (source, hit);
			pending.add (op);
			op.add_callback (request_contact_callback, hit);
		}

		public void exclude_contact (Backend.ContactSource source, Backend.Contact hit) {
			foreach (People.Contact result in contents.get_keys ()) {
				var contacts = contents.get (result);
				if (contacts.remove (hit)) {
					if (contacts.size == 0) {
						contents.remove (result);
						results.remove (result);
						result_discarded (result);
						break;
					} else {
						result.contact_updated ();
					}
				}
			}
		}
		/* End of Internal API*/

		private void hit_found_callback (MultiAsyncSearch<Backend.ContactSource, Backend.Contact> search,
				Backend.ContactSource source, Backend.Contact hit) {
			include_contact (source, hit);
		}

		private void request_contact_callback(AsyncOperation<People.Contact> op,
				void* data) {
			var full_contact = op.get_result ();
			var contact = (Backend.Contact) data;

			Set<Backend.Contact> contacts = contents.get (full_contact);
			if (contacts != null) {
				/* corresponding People.Contact already in the contents */
				contacts.add (contact);
				full_contact.contact_updated ();
			} else {
				/* adding corresponding People.Contact result */
				contacts = new HashSet<Backend.Contact> ();
				contacts.add (contact);
				contents.set (full_contact, contacts);
				results.add (full_contact);
				result_fetched (full_contact);
			}

			pending.remove (op);
			if (finished && (pending.size == 0)) {
				state = QueryResultState.FETCHED;
				finished = false;
			}
		}

		private void done_callback (MultiAsyncSearch<Backend.Contact> search) {
			finished = true;

			if (pending.size == 0) {
				state = QueryResultState.FETCHED;
			}
		}
	}

}
