/* queryable-contact-source-helper.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */
 
using Gee;

using People.Utils;


namespace People {
	/**
	 * QueryableContactSourceHelper is an helper class which intends to provide
	 * emulation of {@link Query} evaluation for backends not supporting it
	 * natively.
	 */
	public class QueryableContactSourceHelper : Object {
		private Map<AsyncSearch<Backend.Contact>, AsyncSearch<Backend.Contact> > pending_searches;
		private Map<AsyncOperation<bool>, AsyncSearch<Backend.Contact> > pending_operations;
		private Map<AsyncSearch<Backend.Contact>, QueryEvaluator> evaluators;

		private Map<AsyncSearch<Backend.Contact>, Set<Backend.Contact> > processing;
		private Set<AsyncSearch<Backend.Contact> > finished_searches;

		public weak Backend.ContactSource source {
			private get;
			private construct set;
		}

		construct {
			pending_searches = new HashMap<AsyncSearch<Backend.Contact>, AsyncSearch<Backend.Contact> > ();
			pending_operations = new HashMap<AsyncOperation<bool>, AsyncSearch<Backend.Contact> > ();
			evaluators = new HashMap<AsyncSearch<Backend.Contact>, QueryEvaluator> ();

			processing = new HashMap<AsyncSearch<Backend.Contact>, Set<Backend.Contact> > ();
			finished_searches = new HashSet<AsyncSearch<Backend.Contact> > ();
		}

		/**
		 * QueryableContactSourceHelper constructor.
		 *
		 * @param source the {@link ContactSource} for which {@link Query}
		 *               evaluation is to be emulated
		 */
		public QueryableContactSourceHelper (Backend.ContactSource source) {
			this.source = source;
		}

		public AsyncSearch<Contact> query (Backend.Query? query) {
			var dump_search = ((Backend.DumpableContactSource) source).dump ();
			dump_search.hit_found += hit_found_callback;
			dump_search.done += done_callback;

			var query_search = new AsyncSearch<Backend.Contact> ();
			pending_searches.set (dump_search, query_search);

			// TODO: use a factory to get the evaluator
			var evaluator = new QueryEvaluator (query);
			evaluators.set (query_search, evaluator);

			dump_search.start ();
			return query_search;
		}

		private void hit_found_callback(AsyncSearch<Backend.Contact> dump_search,
				Backend.Contact contact) {
			var query_search = pending_searches.get (dump_search);

			if (! processing.contains (query_search)) {
				processing.set (query_search, new HashSet<Backend.Contact> ());
			}
			processing.get (query_search).add (contact);

			var op = evaluators.get (query_search).evaluate (contact);
			pending_operations.set (op, query_search);

			op.add_callback (evaluate_callback, contact);
		}

		private void evaluate_callback (AsyncOperation<bool> op, void* data) {
			var contact = (Backend.Contact) data;
			var search = pending_operations.get (op);
			processing.get (search).remove (contact);

			bool result = op.get_result ();
			if (result) {
				search.add_hit (contact);
			}

			if (finished_searches.contains (search) &&
					(processing.get (search).size == 0)) {
				finished_searches.remove (search);
				search.finish ();
			}

			pending_operations.remove (op);
		}

		private void done_callback(AsyncSearch<Backend.Contact> search) {
			var query_search = pending_searches.get (search);
			pending_searches.remove (search);

			if (processing.contains (query_search)) {
				if (processing.get (query_search).size != 0) {
					finished_searches.add (query_search);
				} else {
					query_search.finish ();
				}
			} else {
				query_search.finish ();
			}
		}
	}
}
