using Gee;

using People.Utils;


namespace People.Test {

	class AddressBook : TestIt.Suite {
		private Backend.ContactSource source0;
		private Backend.ContactSource source1;

		private uint number_of_results_fetched;
		private bool state_changed_into_fetched;
		private bool state_changed_into_fetching;

		private People.AddressBook address_book;

		protected override void initialize () {
			name = "People::AddressBook";
			add_test_case ("dump", test_query_none);
			add_test_case ("empty query", test_query_empty);
			add_test_case ("custom query", test_query_custom);
			add_test_case ("whole query", test_query_whole);
		}

		protected override void set_up () {
			/* source 0 */
			var contacts0 = new HashSet<Backend.Contact> ();

			var first0 = new string[] {"Ali", "Johann", "Monchieu"};
			var last0 = new string[] {"Sabil", "Prieur", "Baboul"};
			for(int i = 0; i < first0.length; i++) {
				var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
				fields.set ("identity.name.first", first0[i]);
				fields.set ("identity.name.last", last0[i]);
				var c = new Backend.Dummy.Contact ("c0%d".printf (i), fields, contacts0, false);
				contacts0.add (c);
			}

			source0 = new Backend.Dummy.ContactSource (contacts0, false);

			/* source 1 */
			var contacts1 = new HashSet<Backend.Contact> ();

			var first1 = new string[] {"ali", "Bu", "Janne"};
			var last1 = new string[] {"sabil", "Bronze", "Doe"};
			var email1 = new string[] {"asabil@gmail.com", "jprieur@gmail.com", "jdoe@unknown.uk"};
			for(int i = 0; i < first1.length; i++) {
				var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
				fields.set ("identity.name.first", first1[i]);
				fields.set ("identity.name.last", last1[i]);
				fields.set ("communication.email.home", email1[i]);
				var c = new Backend.Dummy.Contact ("c1%d".printf (i), fields, contacts1, true);
				contacts1.add (c);
			}

			source1 = new Backend.Dummy.ContactSource (contacts1, true);

			/* sources */
			var map = new Gee.HashMap<string, Backend.ContactSource> (str_hash, str_equal);
			map.set ("source0", new ProxyContactSource (source0));
			map.set ("source1", new ProxyContactSource (source1));
			var sources = new BiMap<string, Backend.ContactSource> (map, str_hash, str_equal);

			/* meta contacts definition */
			var merged_contacts = new Gee.HashSet<Gee.List<string> > ();
			var merge1 = new Gee.ArrayList<string> (str_equal);
			merge1.add ("source0:c00");
			merge1.add ("source0:c02");
			merge1.add ("source1:c10");
			merged_contacts.add (merge1);
			var merge2 = new Gee.ArrayList<string> (str_equal);
			merge2.add ("source0:c01");
			merge2.add ("source1:c11");
			merged_contacts.add (merge2);

			/* AddressBook construction */
			address_book = new People.AddressBook (sources, merged_contacts);

			number_of_results_fetched = 0;
			state_changed_into_fetching = false;
			state_changed_into_fetched = false;
		}

		protected override void tear_down () {
			source0 = null;
			source1 = null;

			address_book = null;
		}

		private void test_query_none () {
			QueryResult result = address_book.query (null);
			result.notify["state"] += state_changed_callback;
			result.result_fetched += result_fetched_callback;

			assert_true (result.state == QueryResultState.IDLE);

			result.fetch ();
			async_wait ();

			assert_true (state_changed_into_fetching);
			assert_true (state_changed_into_fetched);
			assert_true (number_of_results_fetched == 3);
		}

		private void test_query_empty () {
			string query = "(CONTAINS identity.name.first \"something_not_appearing_anywhere\")";
			QueryResult result = address_book.query (query);
			result.notify["state"] += state_changed_callback;
			result.result_fetched += result_fetched_callback;

			assert_true (result.state == QueryResultState.IDLE);

			result.fetch ();
			async_wait ();

			assert_true (state_changed_into_fetching);
			assert_true (state_changed_into_fetched);
			assert_true (number_of_results_fetched == 0);
		}

		private void test_query_custom () {
			string query = "(OR (EQUALS identity.name.first \"Ali\") (CONTAINS communication.email.home \"gmail.com\")))";
			QueryResult result = address_book.query (query);
			result.notify["state"] += state_changed_callback;
			result.result_fetched += result_fetched_callback;

			assert_true (result.state == QueryResultState.IDLE);

			result.fetch ();
			async_wait ();

			assert_true (state_changed_into_fetching);
			assert_true (state_changed_into_fetched);
			assert_true (number_of_results_fetched == 2);
		}

		private void test_query_whole () {
			string query = "(CONTAINS identity.name.first \"a\")";
			QueryResult result = address_book.query (query);
			result.notify["state"] += state_changed_callback;
			result.result_fetched += result_fetched_callback;

			assert_true (result.state == QueryResultState.IDLE);

			result.fetch ();
			async_wait ();

			assert_true (state_changed_into_fetching);
			assert_true (state_changed_into_fetched);
			assert_true (number_of_results_fetched == 3);
		}

		private void result_fetched_callback (QueryResult result, Contact contact) {
			number_of_results_fetched++;
		}

		private void state_changed_callback (QueryResult result, ParamSpec spec) {
			if (result.state == QueryResultState.FETCHING) {
				state_changed_into_fetching = true;
			} else if (result.state == QueryResultState.FETCHED) {
				state_changed_into_fetched = true;
				async_break ();
			}
		}

		static int main(string[] args) {
			var suite = new AddressBook();
			return suite.run();
		}
	}
}
