
namespace People.Test {

	class ConfigXml : TestIt.Suite {
		private string file_path = "/tmp/people-test-config-xml.xml";
		private People.ConfigXml config;

		protected override void initialize () {
			name = "People::ConfigXml";
			add_test_case ("list contact sources", test_list_contact_sources);
			add_test_case ("get contact source class", test_get_contact_source_class);
			add_test_case ("get local DB params", test_get_local_db_parameters);
			add_test_case ("get local Keyfile params", test_get_local_keyfile_parameters);
			add_test_case ("get foo bar params", test_get_foo_bar_parameters);
			add_test_case ("list address books", test_list_address_books);
			add_test_case ("get address book sources", test_get_address_book_sources);
			add_test_case ("get address book meta contacts", test_get_address_book_meta_contacts);
		}

		protected override void set_up () {
			var file = FileStream.open(file_path, "w");
			file.puts("<people>\n");
			file.puts("    <contact-sources>\n");
			file.puts("        <contact-source id=\"local-db\" class=\"sqlite\">\n");
			file.puts("            <param name=\"db-path\">local.db</param>\n");
			file.puts("        </contact-source>\n");

			file.puts("        <contact-source id=\"local-keyfile\" class=\"keyfile\">\n");
			file.puts("            <param name=\"file-path\">local.ini</param>\n");
			file.puts("        </contact-source>\n");
			file.puts("    </contact-sources>\n");

			file.puts("    <address-books>\n");
			file.puts("        <address-book id=\"main\">\n");
			file.puts("            <contact-source id=\"local-db\"/>\n");
			file.puts("            <contact-source id=\"local-keyfile\"/>\n");
			file.puts("            <meta-contact>");
			file.puts("                <contact source=\"local-db\" id=\"foo\"/>");
			file.puts("                <contact source=\"local-keyfile\" id=\"bar\"/>");
			file.puts("            </meta-contact>");
			file.puts("            <meta-contact>");
			file.puts("                <contact source=\"local-keyfile\" id=\"boo\"/>");
			file.puts("                <contact source=\"local-db\" id=\"far\"/>");
			file.puts("            </meta-contact>");
			file.puts("        </address-book>\n");
			file.puts("    </address-books>\n");
			file.puts("</people>\n");
			file = null;
			config = new People.ConfigXml (file_path, Environment.get_tmp_dir ());
		}

		protected override void tear_down () {
			config = null;
			FileUtils.unlink (file_path);
		}

		private void test_list_contact_sources() {
			Gee.Set<string> sources = new Gee.HashSet<string> (str_hash, str_equal);
			sources.add ("local-db");
			sources.add ("local-keyfile");

			foreach (string source_id in config.list_contact_sources()) {
				assert_true (sources.contains(source_id));
			}
			assert_true (config.list_contact_sources().size == sources.size);
		}

		private void test_get_contact_source_class() {
			assert_true ((string) config.get_contact_source_class("local-db") == "sqlite");
			assert_true ((string) config.get_contact_source_class("local-keyfile") == "keyfile");
			assert_true (config.get_contact_source_class("foo-bar") == null);
		}

		private void test_get_local_db_parameters() {
			var param = config.get_contact_source_parameters("local-db");
			assert_true (param.size == 1);
			assert_true (param.contains("db-path"));
			assert_true (param.get("db-path").has_suffix ("local.db"));
		}

		private void test_get_local_keyfile_parameters() {
			var param = config.get_contact_source_parameters("local-keyfile");
			assert_true (param.size == 1);
			assert_true (param.contains("file-path"));
			assert_true (param.get("file-path").has_suffix ("local.ini"));
		}

		private void test_get_foo_bar_parameters() {
			assert_true (config.get_contact_source_parameters("foo-bar") == null);
		}

		private void test_list_address_books() {
			Gee.Set<string> books = new Gee.HashSet<string> (str_hash, str_equal);
			books.add ("main");

			foreach (string book_id in config.list_address_books()) {
				assert_true (books.contains(book_id));
			}
			assert_true (config.list_address_books().size == books.size);
		}

		private void test_get_address_book_sources() {
			string[] sources = new string[] {"local-db", "local-keyfile"};
			uint i = 0;

			assert_true (config.get_address_book_sources("main").size == sources.length);

			if (config.get_address_book_sources("main").size == sources.length) {
				foreach (string source_id in config.get_address_book_sources("main")) {
					assert_true (sources[i] == source_id);
					i++;
				}
			}
		}

		private void test_get_address_book_meta_contacts () {
			var meta_contacts = config.get_address_book_meta_contacts ("main");

			assert_true (meta_contacts.size == 2);
			foreach (Gee.List<string> ids in meta_contacts) {
				assert_true (ids.size == 2);
				assert_true ((ids.contains ("local-db:foo") && ids.contains ("local-keyfile:bar")) ||
						(ids.contains ("local-keyfile:boo") && ids.contains ("local-db:far")));
			}
		}

		static int main(string[] args) {
			var suite = new ConfigXml();
			return suite.run();
		}
	}
}
