using Gee;

using People;
using People.Utils;

namespace People.Test {
	class QueryResultManager : TestIt.Suite {
		private Backend.Dummy.ContactSource source0;
		private Backend.Dummy.ContactSource source1;
		private People.AddressBook address_book;
		private People.ContactManager contact_manager;
		private People.QueryResultManager manager;

		private bool result_fetched_callback_called;
		private bool result_discarded_callback_called;

		protected override void initialize () {
			name = "People::QueryResultManager";
			add_test_case ("dump query result lifecycle", test_dump);
			add_test_case ("query query result lifecycle", test_query);
			add_test_case ("dump fetch contact", test_dump_fetch_contact);
			add_test_case ("query fetch contact", test_query_fetch_contact);
			add_test_case ("dump remove contact", test_dump_discard_contact);
		}

		protected override void set_up () {
			/* source 0 */
			var contacts0 = new HashSet<Backend.Contact> ();

			var first0 = new string[] {"Ali", "Johann", "Monchieu"};
			var last0 = new string[] {"Sabil", "Prieur", "Baboul"};
			for(int i = 0; i < first0.length; i++) {
				var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
				fields.set ("identity.name.first", first0[i]);
				fields.set ("identity.name.last", last0[i]);
				var c = new Backend.Dummy.Contact ("c0%d".printf (i), fields, contacts0, false);
				contacts0.add (c);
			}

			source0 = new Backend.Dummy.ContactSource (contacts0, false);

			/* source 1 */
			var contacts1 = new HashSet<Backend.Contact> ();

			var first1 = new string[] {"ali", "Bu", "Janne"};
			var last1 = new string[] {"sabil", "Bronze", "Doe"};
			var email1 = new string[] {"asabil@gmail.com", "jprieur@gmail.com", "jdoe@unknown.uk"};
			for(int i = 0; i < first1.length; i++) {
				var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
				fields.set ("identity.name.first", first1[i]);
				fields.set ("identity.name.last", last1[i]);
				fields.set ("communication.email.home", email1[i]);
				var c = new Backend.Dummy.Contact ("c1%d".printf (i), fields, contacts1, true);
				contacts1.add (c);
			}

			source1 = new Backend.Dummy.ContactSource (contacts1, true);

			/* sources */
			var map = new Gee.HashMap<string, Backend.ContactSource> (str_hash, str_equal);
			map.set ("source0", new ProxyContactSource (source0));
			map.set ("source1", new ProxyContactSource (source1));
			var sources = new BiMap<string, Backend.ContactSource> (map, str_hash, str_equal);

			/* meta contacts definition */
			var merged_contacts = new Gee.HashSet<Gee.List<string> > ();
			var merge1 = new Gee.ArrayList<string> (str_equal);
			merge1.add ("source0:c00");
			merge1.add ("source0:c02");
			merge1.add ("source1:c10");
			merge1.add ("source0:c03");
			merged_contacts.add (merge1);
			var merge2 = new Gee.ArrayList<string> (str_equal);
			merge2.add ("source0:c01");
			merge2.add ("source1:c11");
			merged_contacts.add (merge2);

			/* AddressBook construction */
			address_book = new People.AddressBook (sources, merged_contacts);
			contact_manager = new People.ContactManager (address_book, merged_contacts);

			manager = new People.QueryResultManager (address_book, contact_manager);
			result_fetched_callback_called = false;
			result_discarded_callback_called = false;
		}

		protected override void tear_down () {
			manager = null;
		}

		private void test_dump () {
			QueryResult result;

			assert_true (manager.queries.size == 0);
			assert_true (manager.full.size == 0);

			result = manager.query (null);
			assert_true (manager.queries.size == 0);
			assert_true (manager.full.size == 1);

			result = null;
			assert_true (manager.queries.size == 0);
			assert_true (manager.full.size == 0);
		}

		private void test_query () {
			QueryResult result;

			assert_true (manager.queries.size == 0);
			assert_true (manager.full.size == 0);

			string query = "(CONTAINS identity.name.first \"a\")";
			result = manager.query (query);
			assert_true (manager.queries.size == 1);
			assert_true (manager.full.size == 0);

			result = null;
			assert_true (manager.queries.size == 0);
			assert_true (manager.full.size == 0);
		}

		private void test_dump_fetch_contact () {
			QueryResult result;

			result = manager.query (null);
			result.notify["state"] += result_state_changed_callback;

			result.fetch ();
			async_wait ();
			assert_true (result.results.size == 3);

			result.result_fetched += result_fetched_callback;
			var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			var contact = new Backend.Dummy.Contact ("c04", fields, source0.contacts, false);

			assert_true (source0.add_contact (contact));
			assert_true (result_fetched_callback_called);
			assert_true (result.results.size == 4);
		}

		private void test_query_fetch_contact () {
			QueryResult result;

			string query = "(EQUALS identity.name.last \"Interpol\")";
			result = manager.query (query);
			result.notify["state"] += result_state_changed_callback;

			result.fetch ();
			async_wait ();
			assert_true (result.results.size == 0);

			result.result_fetched += result_fetched_callback;
			var fields0 = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			fields0.set ("identity.name.last", "Interpol");
			var contact0 = new Backend.Dummy.Contact ("c05", fields0, source0.contacts, false);

			assert_true (source0.add_contact (contact0));
			assert_true (result_fetched_callback_called);
			assert_true (result.results.size == 1);

			var fields1 = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			fields1.set ("identity.name.last", "Interpul");
			var contact1 = new Backend.Dummy.Contact ("c06", fields1, source1.contacts, false);

			result_fetched_callback_called = false;
			assert_true (source0.add_contact (contact1));
			assert_true (!result_fetched_callback_called);
			assert_true (result.results.size == 1);
		}

		private void test_dump_discard_contact () {
			QueryResult result;

			result = manager.query (null);
			result.notify["state"] += result_state_changed_callback;

			result.fetch ();
			async_wait ();
			assert_true (result.results.size == 3);

			result.result_discarded += result_discarded_callback;
			var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			var contact = new Backend.Dummy.Contact ("c12", fields, source1.contacts, false);

			assert_true (source1.remove_contact (contact));
			assert_true (result_discarded_callback_called);
			assert_true (result.results.size == 2);

			result_discarded_callback_called = false;
			contact = new Backend.Dummy.Contact ("c01", fields, source0.contacts, false);
			assert_true (source0.remove_contact (contact));
			assert_true (!result_discarded_callback_called);
			assert_true (result.results.size == 2);

			result_discarded_callback_called = false;
			contact = new Backend.Dummy.Contact ("c11", fields, source1.contacts, false);
			assert_true (source1.remove_contact (contact));
			assert_true (result_discarded_callback_called);
			assert_true (result.results.size == 1);
		}

		private void result_state_changed_callback (QueryResult result,
				ParamSpec spec) {
			if (result.state == QueryResultState.FETCHED) {
				async_break ();
			}
		}

		private void result_fetched_callback (QueryResult result, Contact contact) {
			result_fetched_callback_called = true;
		}

		private void result_discarded_callback (QueryResult result, Contact contact) {
			result_discarded_callback_called = true;
		}

		static int main (string[] args) {
			var suite = new QueryResultManager ();
			return suite.run ();
		}
	}
}
