using Gee;

using People;
using People.Utils;


namespace People.Test {

	class NotQueryableContactSource : Object, Backend.ContactSource,
			Backend.DumpableContactSource {
		public Set<Backend.Contact> contacts {
			private get;
			private construct set;
		}

		public Backend.ContactSourceFlags flags {
			get;
			protected set;
			default = Backend.ContactSourceFlags.NONE;
		}

		public NotQueryableContactSource (Set<Backend.Contact> contacts) {
			this.contacts = contacts;
		}

		public AsyncOperation<Backend.Contact> request_contact (string? id = null) {
			return new AsyncOperation<Backend.Contact> ();
		}

		/* DumpableContactSource interface */
		public AsyncSearch<Backend.Contact> dump() {
			var search = new AsyncSearch<Backend.Contact> ();

			foreach (Backend.Contact contact in contacts)
				search.add_hit (contact);
			search.finish ();

			return search;
		}
		/* end of DumpableContactSource interface */
	}

	private class DummyContact : Backend.BaseContact {
		private static Backend.ContactFieldSpec default_field_spec =
				new Backend.ContactFieldSpec(Backend.ContactFieldFlags.NONE);

		public Map<string, string> fields {
			private get;
			private construct set;
		}

		public DummyContact (string id, Map<string, string> fields) {
			this.id = id;
			this.fields = fields;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields() {
			return sync_result (fields.get_keys ());
		}

		public override AsyncOperation<Gee.Set<string> > get_definable_fields() {
			return sync_result (new HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields() {
			return sync_result (new HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<Backend.ContactFieldSpec> get_field_spec(string field_name) {
			return sync_result (default_field_spec);
		}

		public override AsyncOperation<Gee.Map<string, string> >
				get_fields(Gee.Set<string> fields) {
			var result = new HashMap<string, string> (str_hash, str_equal, str_equal);
			foreach (string field in fields) {
				result.set (field, this.fields.get (field));
			}
			return sync_result (result);
		}

		public override AsyncOperation<Gee.Set<string> >
				set_fields(Gee.Map<string, string> fields) {
			return sync_result (new HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<bool> @delete() {
			return sync_result_bool (false);
		}
	}

	class QueryableContactSourceHelper : TestIt.Suite {
		private Backend.ContactSource source;
		private People.QueryableContactSourceHelper helper;

		private AsyncSearch<Backend.Contact> search;

		private Set<Backend.Contact> found_contacts;
		private bool hit_found;
		private bool done;

		protected override void initialize () {
			name = "People::QueryableContactSourceHelper";
			add_test_case ("no selection", test_no_selection);
			add_test_case ("custom selection", test_custom_selection);
			add_test_case ("whole selection", test_whole_selection);
		}

		protected override void set_up () {
			var contacts = new HashSet<Backend.Contact> ();

			var fields0 = new HashMap<string, string> (str_hash, str_equal, str_equal);
			fields0.set ("identity.name.first", "Monsieur");
			fields0.set ("identity.name.last", "Baboul");
			fields0.set ("communication.email.home", "monsieur.baboul@live.com");
			contacts.add (new DummyContact ("0", fields0));

			var fields1 = new HashMap<string, string> (str_hash, str_equal, str_equal);
			fields1.set ("identity.name.first", "Monsieur");
			fields1.set ("identity.name.last", "Baboul");
			fields1.set ("communication.email.home", "monsieur.baboul@gmail.com");
			contacts.add (new DummyContact ("1", fields1));

			var fields2 = new HashMap<string, string> (str_hash, str_equal, str_equal);
			fields2.set ("identity.name.first", "Baboul");
			fields2.set ("identity.name.last", "Baboul");
			fields2.set ("communication.email.home", "monsieur.baboul@gmail.com");
			contacts.add (new DummyContact ("2", fields2));

			var fields3 = new HashMap<string, string> (str_hash, str_equal, str_equal);
			fields3.set ("identity.name.first", "Iorek");
			fields3.set ("identity.name.last", "Byrnison");
			fields3.set ("communication.email.home", "iorek@armoured-bear.fi");
			contacts.add (new DummyContact ("3", fields3));

			source = new NotQueryableContactSource (contacts);
			helper = new People.QueryableContactSourceHelper (source);

			search = null;

			found_contacts = new HashSet<Backend.Contact> ();
			hit_found = false;
			done = false;
		}

		protected override void tear_down () {
			source = null;
			helper = null;

			search = null;
		}

		private void test_no_selection () {
			/* (CONTAINS identity.name.first "something_not_appearing_anywhere") */
			var q0 = new Gee.ArrayList<Backend.Query> ();
			q0.add (new Backend.FieldQuery ("identity.name.first"));
			q0.add (new Backend.ValueQuery ("something_not_appearing_anywhere"));
			var query = new Backend.CompositeQuery (Backend.QueryOperator.CONTAINS, q0);

			search = helper.query (query);
			search.hit_found += no_hit_found_callback;
			search.done += no_done_callback;
			search.start ();
		}

		private void no_hit_found_callback(People.Utils.AsyncSearch<Backend.Contact> search,
				Backend.Contact contact) {
			found_contacts.add (contact);
		}

		private void no_done_callback(People.Utils.AsyncSearch<string> search) {
			assert_true (found_contacts.size == 0);
		}

		private void test_custom_selection () {
			/* (AND (NOT (EQUALS identity.name.first identity.name.last)
					(CONTAINS communication.email.home "gmail.com"))) */
			var q0 = new Gee.ArrayList<Backend.Query> ();
			var q1 = new Gee.ArrayList<Backend.Query> ();
			var q2 = new Gee.ArrayList<Backend.Query> ();

			q1.add (new Backend.FieldQuery("identity.name.first"));
			q1.add (new Backend.FieldQuery("identity.name.last"));
			q0.add (Backend.Query.not (new Backend.CompositeQuery (Backend.QueryOperator.EQUALS, q1)));

			q2.add (new Backend.FieldQuery ("communication.email.home"));
			q2.add (new Backend.ValueQuery ("gmail.com"));
			q0.add (new Backend.CompositeQuery (Backend.QueryOperator.CONTAINS, q2));

			var query = Backend.Query.and (q0);

			search = helper.query (query);
			search.hit_found += custom_hit_found_callback;
			search.done += custom_done_callback;
			search.start ();
		}

		private void custom_hit_found_callback(People.Utils.AsyncSearch<Backend.Contact> search,
				Backend.Contact contact) {
			found_contacts.add (contact);
		}

		private void custom_done_callback(People.Utils.AsyncSearch<string> search) {
			assert_true (found_contacts.size == 1);
		}

		private void test_whole_selection () {
			/* (CONTAINS identity.name.first "o") */
			var q0 = new Gee.ArrayList<Backend.Query> ();
			q0.add (new Backend.FieldQuery ("identity.name.first"));
			q0.add (new Backend.ValueQuery ("o"));
			var query = new Backend.CompositeQuery (Backend.QueryOperator.CONTAINS, q0);

			search = helper.query (query);
			search.hit_found += whole_hit_found_callback;
			search.done += whole_done_callback;
			search.start ();
		}

		private void whole_hit_found_callback(People.Utils.AsyncSearch<Backend.Contact> search,
				Backend.Contact contact) {
			found_contacts.add (contact);
		}

		private void whole_done_callback(People.Utils.AsyncSearch<string> search) {
			assert_true (found_contacts.size == 4);
		}

		static int main (string[] args) {
			var suite = new QueryableContactSourceHelper ();
			return suite.run ();
		}
	}

}
